
# phonopy-FHI-aims.py - phonopy wrapper for FHI-aims
# Copyright (C) 2009-2011 Joerg Meyer (jm)
# Contributions and testing by Christian Carbogno (cc)
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
# * Neither the name of the phonopy project nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# 2016/08/29  togo: Known bug. Support for modulation output doesn't work.
#
# 2016/08/29  togo: Minor fix to follow the change of Phonopy instantiation
#                   Now phonopy_obj.generate_displacements(distance=some_value)
#                   must be called after the Phonopy instantiation.
#
# 2014/09/23  christian.carbogno && shanghui: adding write_eigenfrequencies_and_eigenvectors_at_gamma 
#                                            (both non-nac and nac)                      
#
# 2014/09/22 shanghui: adding support for modulation output 
#                      (modulation setting in control.in)                      
#
# 2014/02/16 togo: Minor fix to follow the change of parse_BORN
# 2014/01/31 togo: Follow the changes of Phonopy and Mesh classes (phonopy-1.8.2)
#
# 2014/01/09 togo: Follow the changes of Mesh, BandStructure, and Animation
#                 classes (phonopy-1.8.0)
#
# 2011/11/13 jm : included support for initial magnetic moments -
#                 (initial_moment tag in geometry.in)
#
# 2011/06/06 jm : polished some formating of numbers printed in output
#                 added support for atom_frac tag in phonopy.interface.FHIaims
#                 verified that 
#                    phonon supercell  -2 1 4  2 -1 4  2 1 -4
#                 (now) yields the correct 2x4x1 mutliple (containing 64 atoms) 
#                 of the smallest simple cubic cell (containing 8 atoms) of a diamond structure
#                 (i.e. two fcc lattices shifted by 0.25*(1,1,1) with respect to each, primitive cell with 8 atoms)
#                 after 
#                    phonopy.structure.cells.get_surrounding_frame()
#                 has been updated
#
# 2011/02/04 jm : adapted to >=phonopy-0.9.2:
#                 IO functionality required for FHI-aims now rewritten in phonopy.FHIaims
#                 phonopy.ASE (and phonopy.ASE_FHIaims) removed since no longer needed
#                 adjusted to argument changes in write_arc and write_xyz
#
# 2010/07/23 jm : made qdensity mesh valued (optional - keeping backward compatibility)
#                 added frequency unit meV
#                 fixed wrong unit output for frequencies at Gamma
#
# 2010/06/10 jm : ported to phonopy-0.9.1.2:
#          & cc   adapted to extensions of dynamical_matrix with respect to non-analytical corrections
#                 added support for animation infrastructure
#                 moved several options to control.in
#
# 2010/05/12 jm : ported to phonopy-0.9.1:
#                 adapted to split of dos array into the two seperate omega, dos arrays in TotalDOS class
#
# 2010/04/10 jm : ported to phonopy-0.9.0
#                 added command line option -y to write yaml files which can be used with
#                 'native' post-processing tools supplied by phonopy
#                 added command line option -e to also calculate eigenvectors 
#                 (which are output to the yaml files only at present if combined with -y option)
#
# 2010/03/23 jm : added command line option -n to include non-analytical terms based on polarizabilities
#                 read from external file (VASP BORN format at present)
#                 removed 'hack' to use q-point distances calculated in BandStructure - now entirely done
#                 in post_process_band
#
# 2010/03/20 jm : ported to phonopy-0.7.4
#                 adapted for ASE-FHI-aims module (containing all FHI-aims specific ASE routines)
#                 exploit symmetry in q-point meshes by default (i.e. set default of symmetry option to True)
#                 Greek letters now substituted for corresponding English letter names as labels in dispersion plots 
#                 (can be disabled via new command line option -g)
#
# 2010/03/02 jm : one minor syntax change to be compatible with Python 2.5.2
#
# 2010/02/26 jm : switched to read_aims_output (instead of detour via aims calculator)
#                 integrated into phonopy-0.7.3
#
# 2010/02/22 jm : support for supercell matrices
#                 optional phonon subtags (according to FHI-aims) manual are now also treated as optional here
#
# 2010/02/19 jm : added new format of force constants matrix (aka Hessian) output 
#                 as required by thermodynamic integration code of Christian Carbogno 
#
# 2010/02/18 jm : included correct index mapping from primitive to supercell in write_Hessian()
#
# 2010/02/08 jm : added output of Hessian aka force constant matrix via command line option -H
#
# 2009/12/27 jm : initial version


class Control:
    def __init__(self, file=None):
        self.phonon = {}
        self.phonon["supercell"] = []
        self.phonon["displacement"] = 0.001
        self.phonon["symmetry_thresh"] = 1E-6
        self.phonon["frequency_unit"] = "cm^-1"
        self.phonon["nac"] = {}
        self.phonon["hessian"] = []
        self.phonon["band"] = []
        self.phonon["dos"] = {}
        self.phonon["free_energy"] = {}
        self.phonon["animation"] = []
        self.phonon["modulations"] = {}
        if file is None:
            self.file = "control.in"
        else:
            self.file = file
        self.read_phonon()
        
    def read_phonon(self):
        f = open(self.file, 'r')
        try:
            for line in f:
                if not line:
                    break
                fields = line.split()
                nfields = len(fields)
                if (nfields > 2) and (fields[0] == "phonon"):
                    if (fields[1] == "supercell"):
                        if (nfields >= 11):
                            s = map(int, fields[2:11])
                            Smat = numpy.array(s).reshape(3,3)
                        elif (nfields >= 5):
                            s = map(int, fields[2:5])
                            Smat = numpy.diag(s)
                        else:
                            raise Exception("invalid supercell")
                        det_Smat = numpy.linalg.det(Smat)
                        if (det_Smat == 0):
                            raise Exception("supercell matrix is not invertible")
                        # this consistency check is not strictly necessary, since int function above used to transform the input 
                        # already throws an exception when decimal numbers are encountered
                        # keep for consistency (and future changes to that spot?) nevertheless...
                        elif (abs(det_Smat-round(det_Smat)) > 1.0e-6):
                            raise Exception("determinant of supercell differs from integer by more than numerical tolerance of 1.0e-6")
                        self.phonon["supercell"] = s
                    if (fields[1] == "displacement"):
                        self.phonon["displacement"] = float(fields[2])
                    if (fields[1] == "symmetry_thresh"):
                        self.phonon["symmetry_thresh"] = float(fields[2])
                    if (fields[1] == "frequency_unit"):
                        self.phonon["frequency_unit"] = fields[2]
                    if (fields[1] == "nac") and (len(fields) >= 4):
                        if (len(fields) >= 7):
                            delta = tuple(map(float, fields[4:7]))
                        else: 
                            delta = (0.1, 0.1, 0.1)
                        if (delta[0] == 0.0) and (delta[1] == 0.0) and (delta[2] == 0.0):
                            raise Exception("evaluation of frequencies with non-analytic corrections must be shifted by delta away from Gamma")
                        parameters = { "file" : fields[2],
                                       "method" : fields[3].lower(),
                                       "delta" : delta }
                        self.phonon["nac"].update(parameters)
                    if (fields[1] == "hessian"):
                        self.phonon["hessian"] = fields[2:]
                    if (fields[1] == "band") and (len(fields) >= 11):
                        parameters = { "kstart" : map(float, fields[2:5]),
                                       "kend" : map(float, fields[5:8]),
                                       "npoints" : int(fields[8]),
                                       "startname" : fields[9],
                                       "endname" : fields[10] }
                        self.phonon["band"].append(parameters)
                    if (fields[1] == "dos") and (len(fields) >= 7):
                        parameters = { "fstart" : float(fields[2]),
                                       "fend" : float(fields[3]),
                                       "fpoints" : int(fields[4]),
                                       "broad" : float(fields[5]),
                                       "qdensity" : map(int, fields[6:]) }
                        self.phonon["dos"].update(parameters)
                    if (fields[1] == "free_energy") and (len(fields) >= 6):
                        parameters = { "Tstart" : float(fields[2]),
                                       "Tend" : float(fields[3]),
                                       "Tpoints" : int(fields[4]),
                                       "qdensity" : map(int, fields[5:]) }
                        self.phonon["free_energy"].update(parameters)
                    if (fields[1] == "animation") and (len(fields) >= 12):
                        parameters = { "q" : map(float, fields[2:5]),
                                       "band" : int(fields[5]),
                                       "amp" : float(fields[6]),
                                       "div" : int(fields[7]),
                                       "shift" : map(float, fields[8:11]),
                                       "files" : fields[11:] }
                        self.phonon["animation"].append(parameters)
                    if (fields[1] == "modulations") :
                        parameters = { "q" : map(float, fields[2:5]),
                                       "supercell" : map(int,fields[5:8]), 
                                       "delta" : float(fields[8]) }
                        self.phonon["modulations"].update(parameters)

        except Exception:
            print line,
            print "|-> line triggered exception: " + str(Exception)
            raise
        # supercell is mandatory for all what follows
        if not self.phonon["supercell"]:
            raise Exception("no supercell specified in %s" % self.file)
        f.close()
        
    def print_phonon(self, prefix=""):
        s = self.phonon["supercell"]
        if (len(s) == 3):
            print prefix + ( "phonon supercell  %d %d %d" % tuple(s) )
        elif (len(s) == 9):
            print prefix + ( "phonon supercell   %d %d %d  %d %d %d  %d %d %d" % tuple(s) )
        print prefix + ( "phonon displacement %f" % self.phonon["displacement"] )
        print prefix + ( "phonon symmetry_thresh %g" % self.phonon["symmetry_thresh"] )
        print prefix + ( "phonon frequency unit %s" % self.phonon["frequency_unit"] )
        nac = self.phonon["nac"]
        if (nac):
            print prefix + ( "phonon nac %s %s" % (nac["file"], nac["method"]) )
        hess = self.phonon["hessian"]
        if (hess):
            print prefix + ( "phonon %s" % " ".join(hess) )
        for nb in range(len(self.phonon["band"])):
            b = self.phonon["band"][nb]
            print prefix + ( "phonon band  %f %f %f  %f %f %f  %d  %s  %s" % \
                             tuple(b["kstart"] + b["kend"] + [b["npoints"]] + [b["startname"]] + [b["endname"]]) )
        d = self.phonon["dos"]
        if (d):
            print prefix + ( "phonon dos %f %f %d %f %s" % \
                             ( d["fstart"], d["fend"], d["fpoints"], d["broad"], \
                               ' '.join(map(str, d["qdensity"])) ) )
        fe = self.phonon["free_energy"]
        if (fe):
            print prefix + ( "phonon free_energy %f %f %d %s" % \
                             ( fe["Tstart"], fe["Tend"], fe["Tpoints"], \
                               ' '.join(map(str, fe["qdensity"])) ) )
        for na in range(len(self.phonon["animation"])):
            anim = self.phonon["animation"][na]
            print prefix + ( "phonon animation  %f %f %f  %d %d %d  %f %f %f  %s" % \
                             tuple(anim["q"] + [anim["band"], anim["amp"], anim["div"]]  
                                             + anim["shift"] + [' '.join(anim["files"])]) )


from phonopy.structure.cells import Primitive
from phonopy.harmonic.forces import Forces
from phonopy.harmonic.force_constants import get_force_constants
def write_Hessian(phonopy_obj, supercell_matrix_inv, set_of_forces):
    cells = (("initial cell", phonopy_obj.unitcell), ("supercell", phonopy_obj.supercell))
    primitive = Primitive(phonopy_obj.supercell, supercell_matrix_inv)
    forces = []
    for (i, disp) in enumerate(phonopy_obj.displacements):
        atom_number = disp[0]
        displacement = disp[1:]
        forces.append(Forces(atom_number, displacement, set_of_forces[i]))
    Hessian = get_force_constants(forces, phonopy_obj.symmetry, phonopy_obj.supercell)
    f = open("phonopy-FHI-aims-Hessian.dat", 'w')
    f.write("### Hessian from phonopy-FHI-aims \n")
    f.write("# \n")
    for (cell_name,cell) in cells:
        f.write("# %s: \n" % cell_name)
        for i in range(3):
            v = cell.get_cell()[i]
            f.write("#   lattice_vector %d   %11.6f % 11.6f %11.6f \n" % (i+1,v[0],v[1],v[2]))
        for n in range(cell.get_number_of_atoms()):
            r = cell.get_positions()[n]
            sym = cell.get_chemical_symbols()[n]
            f.write("#   atom %4d   %11.6f %11.6f %11.6f   %s \n" % (n+1,r[0],r[1],r[2],sym)) 
        f.write("# \n")
    f.write("# force constant matrix elements in eV/(Angstrom)^2 \n")	
    for (j_atom_count,j_atom) in enumerate(primitive.get_primitive_to_supercell_map()):
        for j_coord in range(3):
            for i_atom in range(phonopy_obj.supercell.get_number_of_atoms()):
                for i_coord in range(3):
                    f.write("%18.10e " % Hessian[i_atom,j_atom,i_coord,j_coord])
                f.write("  ")
            f.write("\t # displaced atom %d, coord %d \n" % (j_atom+1,j_coord+1))
    f.close()


from phonopy.harmonic.forces import Forces
from phonopy.harmonic.force_constants import get_force_constants
def write_force_constants(phonopy_obj, set_of_forces):
    cells = (("initial cell", phonopy_obj.unitcell,"#"), ("supercell", phonopy_obj.supercell,""))
    Nsuper = phonopy_obj.supercell.get_number_of_atoms()
    forces = []
    for (i, disp) in enumerate(phonopy_obj.displacements):
        atom_number = disp[0]
        displacement = disp[1:]
        forces.append(Forces(atom_number, displacement, set_of_forces[i]))
    force_constants = get_force_constants(forces, phonopy_obj.symmetry, phonopy_obj.supercell)
    f = open("phonopy-FHI-aims-force_constants.dat", 'w')
    f.write("### Hessian from phonopy-FHI-aims \n")
    f.write("# \n")
    for (cell_name,cell,prefix) in cells:
        f.write("# %s: \n" % cell_name)
        for i in range(3):
            v = cell.get_cell()[i]
            f.write(prefix + ("lattice_vector %11.6f %11.6f %11.6f   %4d \n" % (v[0],v[1],v[2],i+1)))
        for n in range(cell.get_number_of_atoms()):
            r = cell.get_positions()[n]
            sym = cell.get_chemical_symbols()[n]
            f.write(prefix + ("atom           %11.6f %11.6f %11.6f   %3s   %4d \n" % (r[0],r[1],r[2],sym,n+1))) 
        f.write("\n")
    f.write("# \n")
    f.write("# force constant matrix elements in eV/(Angstrom)^2 \n")
    f.write("#  1 2 3 means displaced atom 1, in coord 2, force on atom 3 in this line \n")
    for j_atom in range(Nsuper):
        for j_coord in range(3):
            for i_atom in range(Nsuper):
                fcs = [ force_constants[i_atom,j_atom,i_coord,j_coord] for i_coord in range(3) ]
                indeces = [ j_atom+1, j_coord+1, i_atom+1 ]
                f.write("force_constants %18.10e %18.10e %18.10e      %d %d %d \n" % 
                        (tuple(fcs)+tuple(indeces)))
        f.write("\n")
    f.close()


from phonopy.units import VaspToTHz as AimsToTHz, VaspToCm as AimsToCm, VaspToEv as AimsToEv, THzToCm, THzToEv
AimsFrequencyUnitFactors = { 'cm^-1' : AimsToCm, 'THz' : AimsToTHz, 'meV' : 1E3*AimsToEv } 
# use "\minus" instead of "-" here to avoid down arrows sometimes printed by matplotlib instead
AimsFrequencyUnitLabelsMatplotlib = { 'cm^-1' : "cm$^{\minus 1}$", 'THz' : "THz", 'meV' : "meV" }
AimsFrequencyUnitFactorsToTHz = { 'cm^-1' : 1.0 / THzToCm, 'THz' : 1.0, 'meV' : 1.0 / THzToEv / 1000 } 

BandStructureLabels = { 'gamma'    : "\Gamma", 
                        'delta'    : "\Delta", 
                        'lambda'   : "\Lambda",
                        'sigma'    : "\Sigma" }
    
from phonopy.phonon.band_structure import BandStructure
def post_process_band(phonopy_obj, parameters, frequency_unit_factor, is_eigenvectors=False, write_yaml=False, do_matplotlib=False, lookup_labels=False):
    bands = []
    # Distances calculated in phonopy.band_structure.BandStructure object
    # are based on absolute positions of q-points in reciprocal space
    # as calculated by using the cell which is handed over during instantiation.
    # Fooling that object by handing over a "unit cell" diag(1,1,1) instead clashes
    # with calculation of non-analytical terms.
    # Hence generate appropriate distances and special k-points list based on fractional
    # coordinates in reciprocal space (to keep backwards compatibility with previous
    # FHI-aims phonon implementation).
    bands_distances = []
    distance = 0.0
    bands_special_points = [distance]
    bands_labels = []
    label = parameters[0]["startname"]
    if lookup_labels:
        bands_labels.append(BandStructureLabels.get(label.lower(),label))
    else:
        bands_labels.append(label)
    for b in parameters:
        kstart = numpy.array(b["kstart"])
        kend = numpy.array(b["kend"])
        npoints = b["npoints"]
        dk = (kend-kstart)/(npoints-1)
        bands.append([(kstart + dk*n) for n in range(npoints)])
        dk_length = numpy.linalg.norm(dk)
        # one long list to simplify output
        for n in range(npoints):
            bands_distances.append(distance + dk_length*n)
        distance += dk_length * (npoints-1)
        bands_special_points.append(distance)
        # assuming that startname is the same as previous endname
        # (i.e. non-interrupted paths!)
        label = b["endname"]
        if lookup_labels:
            bands_labels.append(BandStructureLabels.get(label.lower(),label))
        else:
            bands_labels.append(label)
    bs_obj = BandStructure(bands, 
                           phonopy_obj.dynamical_matrix, 
                           is_eigenvectors=is_eigenvectors,
                           factor=frequency_unit_factor)
    # make band index first index (simpler for bands plotting !)
    eigenvalues = numpy.vstack(bs_obj.get_eigenvalues()).T
    frequencies = numpy.zeros_like(eigenvalues)
    for i, eigenvalues_band in enumerate(eigenvalues):
        frequencies_band = []
        for eigenvalue in eigenvalues_band:
            if eigenvalue < 0:
                frequencies_band.append(-numpy.sqrt(-eigenvalue))
            else: 
                frequencies_band.append(numpy.sqrt(eigenvalue))
        frequencies[i] = numpy.array(frequencies_band)
    fmin = numpy.min(frequencies)
    fmax = numpy.max(frequencies)
    for unit in AimsFrequencyUnitFactors:
        factor = AimsFrequencyUnitFactors[unit]
        if (factor == frequency_unit_factor):
            frequency_unit = unit
            break
    f = open("phonopy-FHI-aims-band_structure.dat", 'w')
    f.write("#\n")
    f.write("# Phonon bands from phonopy-FHI-aims!\n")
    f.write("#\n")
    lattice_real = phonopy_obj.unitcell.get_cell()
    lattice_reciprocal = 2.0 * numpy.pi * numpy.linalg.inv(lattice_real.transpose())
    for i in range(3):
        f.write("# Reciprocal lattice vector %11.6f %11.6f %11.6f \n" % tuple(lattice_reciprocal[i]))
    digits = int( math.ceil( math.log(len(parameters)+1,10) ) ) + 1
    digits_string = "%0" + str(digits) + "d"
    total_qpoints = 0
    total_qpoints_bands = []
    for i, b in enumerate(parameters):
        f.write("#\n")
        distance_start = bands_distances[total_qpoints]        
        f.write( ("# %5s point for band " + digits_string + ", %5s = (%9.5f,%9.5f,%9.5f) will be at real distance = %11.5f \n") % \
                  tuple(["Start"] + [i] + [b["startname"]] + b["kstart"] + [distance_start]) )
        total_qpoints += b["npoints"]
        distance_end = bands_distances[total_qpoints-1]
        f.write( ("# %5s point for band " + digits_string + ", %5s = (%9.5f,%9.5f,%9.5f) will be at real distance = %11.5f \n") % \
                  tuple(["End"] + [i] + [b["endname"]] + b["kend"] + [distance_end]) )                 
        total_qpoints_bands.append(total_qpoints)
    f.write("#\n")
    f.write("# number of phonon branches : %d \n" % len(frequencies))
    f.write("#\n")
    f.write("# q-distance(frac.)  frequencies(%s) \n" % frequency_unit)
    for i, dq in enumerate(bands_distances):    
        f.write("%f " % dq)
        frequencies_at_q = frequencies[:,i]
        for freq in frequencies_at_q:
            f.write(" %11.6f" % (freq*bs_obj.get_unit_conversion_factor()))
        f.write(" \n")
        if (i+1) in total_qpoints_bands:
            f.write("\n")
    f.close()
    if write_yaml:
        bs_obj.write_yaml()
    if do_matplotlib:
        params = { 'axes.labelsize': 20,
                   'xtick.labelsize' : 16,
                   'ytick.labelsize' : 16,
                   'text.fontsize': 24 }
        plt.rcParams.update(params)
        plt.cla()
        for frequencies_band in frequencies:
            plt.plot(bands_distances, frequencies_band*bs_obj.get_unit_conversion_factor(), 'r-')
        plt.xticks( bands_special_points,
                    tuple("$\mathrm{\mathsf{%s}}$" % bands_labels[i] for i in range(len(bands_labels))) )
        plt.xlabel('Wave vector')
        plt.ylabel("Frequency (%s)" % AimsFrequencyUnitLabelsMatplotlib[frequency_unit])
        plt.vlines(bands_special_points[1:-1], *plt.ylim())
        plt.xlim(xmin=0, xmax=bands_distances[-1])
        plt.ylim(ymin=round(fmin*bs_obj.get_unit_conversion_factor()))
        plt.savefig("phonopy-FHI-aims-band_structure.pdf")    

from phonopy.phonon.dos import TotalDos
def post_process_dos(phonopy_obj, mesh_obj, parameters, do_matplotlib=False):
    broad = parameters["broad"]
    dos_obj = TotalDos(mesh_obj, sigma=broad)
    fmin = parameters["fstart"]
    fmax = parameters["fend"] 
    fstep = ( fmax - fmin ) / parameters["fpoints"]
    dos_obj.set_draw_area(fmin, fmax, fstep)
    dos_obj.run()
    for unit in AimsFrequencyUnitFactors:
        factor = AimsFrequencyUnitFactors[unit]
        if (factor == frequency_unit_factor):
            frequency_unit = unit
            break
    f = open("phonopy-FHI-aims-dos.dat", 'w')
    f.write("# Phonon density of states from phonopy-FHI-aims\n")
    f.write("# %20s %20s \n" % ("Frequency (%s)" % frequency_unit, "DOS"))

    omegas, total_dos = dos_obj.get_dos()
    for freq, dos in zip(omegas, total_dos):
        f.write("  %20.10f %20.10f \n" % (freq, dos))
    f.close()
    if do_matplotlib:
        params = { 'axes.labelsize': 20,
                   'xtick.labelsize' : 16,
                   'ytick.labelsize' : 16,
                   'text.fontsize': 24 }
        plt.rcParams.update(params)
        plt.cla()
        plt.plot(omegas, total_dos, 'r-')
        plt.grid(True)
        plt.xlabel("Frequency (%s)" % AimsFrequencyUnitLabelsMatplotlib[frequency_unit])
        plt.ylabel("Density of states (a.u.)")
        plt.yticks([])
        plt.savefig("phonopy-FHI-aims-dos.pdf")

from phonopy.phonon.thermal_properties import ThermalProperties
from phonopy.units import EvTokJmol, Kb as kBoltzmann
def post_process_free_energy(phonopy_obj, mesh_obj, parameters, frequency_unit_to_THz, write_yaml=False, do_matplotlib=False):
    tp_obj = ThermalProperties(mesh_obj.get_frequencies() * frequency_unit_to_THz,
                               weights=mesh_obj.get_weights())
    Tmin = parameters["Tstart"]
    Tmax = parameters["Tend"]
    Tstep = ( Tmax - Tmin) / parameters["Tpoints"]
    tp_obj.run(Tstep, Tmax, Tmin)
    T, F, S, cv = tp_obj.get_thermal_properties()
    kJmolToEv = 1.0 / EvTokJmol
    JmolToEv = kJmolToEv / 1000
    T_aims = T
    F_aims = kJmolToEv * F
    JmolToEv = kJmolToEv / 1000
    TS_aims = T * (JmolToEv * S)
    U_aims = F_aims + TS_aims
    cv_aims = JmolToEv / kBoltzmann * cv    
    f = open("phonopy-FHI-aims-free_energy.dat", 'w')
    f.write("# Phonon free energy and specific heat from phonopy-FHI-aims\n")
    f.write("# Phonon zero point energy = %22.9f eV \n" % (F_aims[0]))
    f.write( ("#%20s " + "%30s"*4 + "\n") %
             ("Temperature (K)", "Free energy (eV/cell)", "Internal energy (eV/cell)",
              "c_v (kB/cell)", "-TS_vib(eV/cell)") )
    for i in range(T.shape[0]):
        f.write( ("%20.3f " + "%30.9f"*4 + "\n") %
                 (T_aims[i], F_aims[i], U_aims[i], cv_aims[i], -TS_aims[i]) )
    f.close()
    if write_yaml:
        tp_obj.write_yaml()
    if do_matplotlib:
        params = { 'axes.labelsize': 20,
                   'xtick.labelsize' : 16,
                   'ytick.labelsize' : 16,
                   'text.fontsize': 24 }
        plt.rcParams.update(params)
        plt.cla()
        plt.plot(T, F_aims, 'r-', label="Free energy (eV/cell)")
        plt.plot(T, U_aims, 'b-', label="Internal energy (eV/cell)")
        plt.plot(T, cv_aims, 'g-', label="c$_\mathrm{v}$ (kB/cell)")
        plt.plot(T, -TS_aims, "k--", label="-TS_vib(eV/cell)")
        plt.legend(loc='best', shadow=True)
        plt.grid(True)
        plt.xlabel("Temperature (K)")
        plt.savefig("phonopy-FHI-aims-free_energy.pdf")



#------------------begin for aims_modulation----------------------------
from phonopy.phonon.modulation import Modulation
from phonopy.phonon.degeneracy import get_eigenvectors
from phonopy.structure.cells import get_supercell
def run_aims_modulation(phonopy_obj):
        f_freq = open("freq.dat", 'w')
        f_norm = open("norm.dat", 'w')
        frequencies = []
        for ph_mode in phonopy_obj._phonon_modes:
            q, band_index, amplitude, argument = ph_mode
            eigvals, eigvecs = get_eigenvectors(
                q,
                phonopy_obj._dm,
                phonopy_obj._ddm,
                perturbation=phonopy_obj._delta_q,
                derivative_order=phonopy_obj._derivative_order,
                nac_q_direction=phonopy_obj._nac_q_direction)
            #print '---------------(1)d(q)----------------------'
            #print eigvecs[:, band_index]
            #print '---------------(2)d(q) expt(iqR)/sqrt(M)----------------------'
            u = phonopy_obj._get_delta(eigvecs[:, band_index], q)
            #print u
            phonopy_obj._eigvecs.append(eigvecs[:, band_index])
            phonopy_obj._eigvals.append(eigvals[band_index])
            #---------shanghui begin freq-------------------------------------
            if eigvals[band_index]  < 0:
                # Safety check for imaginary frequencies:
                neg_value = -numpy.sqrt(-eigvals[band_index])*AimsToCm
                if neg_value < -0.1:
                   print "* WARNING:"
                   print "* Imaginary Eigenfrequencies unphysical in modulation_aims.py"
                frequencies.append(neg_value)
            else:
                frequencies.append(numpy.sqrt(eigvals[band_index])*AimsToCm)  
            #---------shanghui end freq-------------------------------------

            #---------shanghui begin norm-------------------------------------
            norm =  numpy.linalg.norm(numpy.array(u).real)
            u = u/norm 
            f_norm.write("%d%18.12f \n" %(band_index+1, norm*norm))
            #----------shanghui end norm--------------------------------------
            phonopy_obj._delta_modulations.append(u * amplitude)

       
        for freq in frequencies:
            f_freq.write(" %11.6f" %freq)
        f_freq.close()
        f_norm.close()

def write_aims_modulation(self, filename="output_geometry"):
        deltas = []
        for i, delta_positions in enumerate(self._delta_modulations):
            cell = self._get_cell_with_modulation(delta_positions)
            write_aims((filename+"-%03d") % (i+1), cell)
            deltas.append(delta_positions)
    
        sum_of_deltas = numpy.sum(deltas, axis=0)
        no_modulations = numpy.zeros(sum_of_deltas.shape, dtype=complex)
        cell = self._get_cell_with_modulation(no_modulations)
        write_aims(filename+"-orig", cell)

def post_process_write_modulations(phonopy_obj,parameters):
        setting =  {}
        #-----set by reading contrin.in--------
        q = parameters['q']
        setting['dimension'] = parameters['supercell'] 
        amplitude = parameters['delta']

        #-----set to default values------------
        setting['order'] = None # This is derivtive of density matrix for group velocity
        argument=float(0.0) # This is phase factor, we set 0 here

        # add to phonon_modes
        vals=[]
        for band_index in range(3*cell.get_number_of_atoms()): 
            vals.append([q, band_index, amplitude,argument])

        setting['phonon_modes'] = vals 

        phonopy_obj._modulation = Modulation(phonopy_obj._dynamical_matrix,
                                      dimension=setting['dimension'],
                                      phonon_modes=setting['phonon_modes'],
                                      delta_q=None,
                                      derivative_order=None,
                                      nac_q_direction=None,
                                      factor=phonopy_obj._factor)

        run_aims_modulation(phonopy_obj._modulation)

        """Create output_geometry"""
        write_aims_modulation(phonopy_obj._modulation,"output_geometry")
#------------------end for aims_modulation----------------------------


from phonopy.phonon.animation import Animation
def post_process_animation(phonopy_obj, parameters, frequency_unit_factor):
    for anim in parameters:
        anim_obj = Animation(anim["q"],
                             phonopy_obj.dynamical_matrix)
        for file in anim["files"]:
            suffix = os.path.splitext(file)[-1]
            if (suffix == ".ascii"):
                anim_obj.write_v_sim(factor=frequency_unit_factor, filename=file)
            if (suffix == ".arc"):
                anim_obj.write_arc(band_index=anim["band"], amplitude=anim["amp"], 
                                   num_div=anim["div"], filename=file)
            if (suffix == ".xyz_jmol"):
                anim_obj.write_xyz_jmol(factor=frequency_unit_factor, filename=file)
            if (suffix == ".xyz"):
                anim_obj.write_xyz(band_index=anim["band"], amplitude=anim["amp"], 
                                   num_div=anim["div"], 
                                   factor=frequency_unit_factor, filename=file)

from phonopy.units import VaspToEv as AimsToEv
from phonopy.units import VaspToCm as AimsToCm
def write_eigenfrequencies_and_eigenvectors_at_gamma(phonopy_obj):
    # Only Gamma point is of interest
    q = numpy.zeros(3)


    # Check for unit cell == supercell
    # FIXME: CC an unfolding of the gamma frequencies
    #        and eigenvectors might be added later on 
    n_atoms =  phonopy_obj.unitcell.get_number_of_atoms() 
    if  n_atoms !=  phonopy_obj.supercell.get_number_of_atoms():
	print " * WARNING:"
	print " * The unitcell and the supercell are not identical."

    # Open file and write reference equilibrium geometry
    f = open("phonopy-FHI-aims-eigen-frequencies-and-vectors-at-Gamma.dat", 'w')
    f.write("### Eigenfrequencies and Eigenvectors at Gamma from phonopy-FHI-aims \n")
    f.write("# \n")
    f.write("# Equilibrium geometry\n")
    cell = phonopy_obj.unitcell
    for i in range(3):
        v = cell.get_cell()[i]
        f.write("lattice_vector %20.10f %20.10f %20.10f   \n" % (v[0],v[1],v[2]))
    for n in range(n_atoms):
        r = cell.get_positions()[n]
        sym = cell.get_chemical_symbols()[n]
        f.write("atom           %20.10f %20.10f %20.10f   %3s   %4d \n" % (r[0],r[1],r[2],sym,n+1)) 

    # Calculate Eigenvalues and Eigenfrequencies at Gamma
    f.write("\n# \n# Eigenfrequencies at Gamma in cm^-1 (ascending order) \n")


#------------------ shanghui-add for nac_gamma-------------------------------------------
#------------using referenc of phonopy/phonon/band_structure.py (set_band)------------------        
    q_direction= numpy.array([0.5,0.5,0.5]) 
 
    if requested_nac: # at gamma point, you need give a q_dirction, then the nac can be added.
       phonopy_obj.dynamical_matrix.set_dynamical_matrix(q,q_direction)
       dm = phonopy_obj.dynamical_matrix.get_dynamical_matrix()
    else :
       phonopy_obj.dynamical_matrix.set_dynamical_matrix(q)
       dm = phonopy_obj.dynamical_matrix.get_dynamical_matrix()

    frequencies = []
    eigfreq, eigvec =  numpy.linalg.eigh(dm)
#--------------------shanghui-end-add for nac_gamma----------------------------------------------


    # Write eigenvalues to file
    for eig in eigfreq:
            if eig < 0:
    		# Safety check for imaginary frequencies:
		neg_value = -numpy.sqrt(-eig)*AimsToEv*1000
		if neg_value < -0.1:
	           print "* WARNING:"  
	           print "* Imaginary Eigenfrequencies at the Gamma point are highly unphysical."
                frequencies.append(neg_value)
            else:
                frequencies.append(numpy.sqrt(eig)*AimsToCm)  #AimsToEv*1000)
    for i, freq in enumerate(frequencies):
        f.write("frequency %20.10f \t %4d \n" %  ( freq,i+1))

    # Write eigenvectors to file
    f.write("\n#\n# Eigenvectors at Gamma (orthonormalized) \n")
    for i, freq in enumerate(frequencies):
        eigvec_band_i = eigvec[:,i]
    	# Safety check for imaginary eigenvector components:
	if (abs(eigvec_band_i.imag) > 1E-7).any():
	   print "* WARNING:"
	   print "* Imaginary Eigenvectors at the Gamma point are highly unphysical."
			
    	f.write("#\n# Mode %4d  - Atoms 1 to %4d  - Eigenfrequency  %20.10f\n" % (i+1,n_atoms,frequencies[i]))
    	for n in range(n_atoms):
    		f.write("eigenvector %4d %4d  %20.15f %20.15f %20.15f \n" \
		% (i+1,n+1,eigvec_band_i[3*n],eigvec_band_i[3*n+1],eigvec_band_i[3*n+2]))
    f.close()

   

if __name__ == "__main__":

    # suppressing warnings, particularly from imported packages, to keep the output clean
    # -> comment out for debugging purposes!
    import warnings
    warnings.simplefilter('ignore')

    import sys
    import os
    import shutil
    import math
    import numpy

    from optparse import OptionParser

    from phonopy import Phonopy
#    from ase.io.aims import read_aims, write_aims, read_aims_output
    from phonopy.interface.FHIaims import read_aims, write_aims, read_aims_output
    from phonopy.phonon.mesh import Mesh

    print "### phonopy wrapper for FHI-aims ###"
    version = "20111113"
    print "# version " + version
    print "#"

    has_matplotlib = False
    try:
        import matplotlib
        matplotlib.use("PDF")
        import matplotlib.pyplot as plt
        has_matplotlib = True
    except ImportError:
        print "# matplotlib is not available"
        print "#"
    
    usage = "usage: %prog [options] [arguments are ignored] \n" + \
            "       run in directory with control.in and geometry.in files"
    parser = OptionParser(version=version, usage=usage)
    
    parser.add_option("-d", "--data-only",
                      action="store_true", dest="data", default=False,
                      help="only write .dat files (no plots even if matplotlib is available)")
    parser.add_option("-e", "--eigenvectors",
                      action="store_true", dest="eigenvectors", default=False,
                      help="also calculate Eigenvectors for every q-point occuring in current calculation")
    parser.add_option("-g", "--no-greek-labels",
                      action="store_false", dest="greek_labels", default=True,
                      help="turn off replacement of Greek letters for labels in band structure plots")
    parser.add_option("-s", "--no-symmetry",
                      action="store_false", dest="symmetry", default=True,
                      help="turn off symmetry wherever possible (at the moment only for q-point grid)")
    parser.add_option("-G", "--EigVecGamma",
                      action="store_true", dest="EigVecGamma", default=False,
                      help="output Eigenvalues and Eigenvectors at the Gamma point")
    parser.add_option("-y", "--yaml",
                      action="store_true", dest="yaml", default=False,
                      help="write .yaml data files for 'native' phonopy post-processing tools")
    (options, args) = parser.parse_args()
    do_matplotlib = has_matplotlib and not options.data
    
    print "# reading control.in"
    control = Control()
    control.print_phonon("# |   ")
    if (len(control.phonon["supercell"]) == 3):
        supercell_matrix = numpy.diag(control.phonon["supercell"])
    elif (len(control.phonon["supercell"]) == 9):
        supercell_matrix = numpy.array(control.phonon["supercell"]).reshape(3,3)
    supercell_matrix_inv = numpy.linalg.inv(supercell_matrix)
    print "# +-> specified supercell has %.1f times the volume of the initial cell" % numpy.linalg.det(supercell_matrix)
    print "#"

    frequency_unit = control.phonon["frequency_unit"]
    frequency_unit_factor = AimsFrequencyUnitFactors[frequency_unit]
    frequency_unit_to_THz = AimsFrequencyUnitFactorsToTHz[frequency_unit]

    print "# reading geometry.in"
    cell = read_aims("geometry.in")
    if cell.get_magnetic_moments() is not None:
        print "# | initial_moments found and about to be replicated in supercells"
        print "# | WARNING: No consideration in symmetry analysis, i.e."
        print "# |          if initial spin breaks the spatial symmetry,"
        print "# |          this is currently NOT detected."
        print "# |          You thus might want to consider the"
        print "# |          command line option to turn off symmetry."
    print "# generating supercells with displacements"
    phonopy_obj = Phonopy(cell, supercell_matrix, 
                          symprec=control.phonon["symmetry_thresh"],
                          factor=frequency_unit_factor)
    phonopy_obj.generate_displacements(distance=control.phonon["displacement"])
    print "# | Spacegroup: %s" % phonopy_obj.symmetry.get_international_table()
    supercells = phonopy_obj.get_supercells_with_displacements()
    digits = int( math.ceil( math.log(len(supercells)+1,10) ) ) + 1
    directories = []
    for i in range(len(supercells)):
        directories.append( ("phonopy-FHI-aims-displacement-%0" + str(digits) + "d") % (i+1))
    print "#"

    if not any(map(os.path.isdir, directories)):
        for (supercell, directory) in zip(supercells, directories):
            os.mkdir(directory)
            write_aims(os.path.join(directory, "geometry.in"), supercell)
            shutil.copy("control.in", directory)
        print "!   Please calculate forces with FHI-aims for the (supercell) structures"
        print "!   which have been generated in the subdirectories"
        for directory in directories:
            print "! \t %s" % directory
        print "!   redirecting the outputs into"
        print "!	<directory name>.out"
        print "!   in each directory." 
        quit()

    print "# reading forces from"
    set_of_forces = []
    for (directory, supercell) in zip(directories, supercells):
        aims_out = os.path.join(directory, directory + ".out")
        print "# | %s" % aims_out
        if not os.path.isfile(aims_out):
            print "!!! file not found: %s" % aims_out
            sys.exit(1)
        # FHI-aims calculation is supposed to be a single point - 
        # hence only use first structure contained in aims output
        supercell_calculated = read_aims_output(aims_out)
        # sanity check to verify that calculated structures from which forces are read in 
        # are those which are expected
        # cannot use builtin __eq__ of atoms object here because 
        #    - numerical tolerances are required when comparing read in positions and cell vectors
        #    - FHI-aims does not support other than 0 or 3 periodic directions
        tol = 1.0e-6
        if ( (supercell_calculated.get_number_of_atoms() == supercell.get_number_of_atoms()) and
             (supercell_calculated.get_atomic_numbers() == supercell.get_atomic_numbers()).all() and
             (abs(supercell_calculated.get_positions()-supercell.get_positions()) < tol).all() and
             (abs(supercell_calculated.get_cell()-supercell.get_cell()) < tol).all() ):
            # read_aims_output reads in forces from FHI-aims output as list structure, 
            # but further processing below requires numpy array
            forces = numpy.array(supercell_calculated.get_forces())
            drift_force = forces.sum(axis=0)
            print "#   | correcting drift : %11.5f %11.5f %11.5f" % tuple(drift_force)
            for force in forces:
                force -= drift_force / forces.shape[0]
            set_of_forces.append(forces)
        else:
            print "!!! calculated varies from expected supercell in FHI-aims output %s" % aims_out
            sys.exit(2)
    print "#"

    if "phono-perl" in control.phonon["hessian"]:
        print "# writing Hessian matrix (same format as Perl script)"
        write_Hessian(phonopy_obj, supercell_matrix_inv, set_of_forces)
        print "#"
    if "TDI" in control.phonon["hessian"]:
        print "# writing force constants"
        write_force_constants(phonopy_obj, set_of_forces)
        print "#"

    requested_nac = ( control.phonon["nac"] != {} )
    # Since phonopy-0.9.0, first argument only denotes axes of primitive cell relative to input unit cell 
    # (always unit matrix here), i.e. supercell_matrix inversion (as required for Primitive()) is done 
    # in Phonopy.set_post_process() directly.
    print "# preparing calculation of dynamical matrix"
    # phonopy_obj.set_post_process(numpy.eye(3), set_of_forces)
    phonopy_obj.set_forces(set_of_forces)
    phonopy_obj.produce_force_constants()
    if requested_nac:
        from phonopy.hphonopy.file_IO import parse_BORN
        nac_file = control.phonon["nac"]["file"]
        nac_method = control.phonon["nac"]["method"]
        print "# | reading polarizabilities from %s" % nac_file
        nac_params = parse_BORN(phonopy_obj.primitive, filename=nac_file)
        # print "# | setting method for non-analytical terms in dynamical matrix to \'%s\'" % nac_method
        phonopy_obj.set_nac_params(nac_params)
        print "#"

    if requested_nac:
        print "# phonon frequencies at Gamma + %s:" % str(control.phonon["nac"]["delta"])
        q = numpy.zeros(3) + numpy.array(control.phonon["nac"]["delta"])
    else:
        print "# phonon frequencies at Gamma:"
        q = numpy.zeros(3)
    # Since phonopy-0.9.0, Phonopy.get_frequencies already returns frequencies in units 
    # according to (unit) factor relayed at Phonopy object instantiation.
    for i, freq in enumerate(phonopy_obj.get_frequencies(q)):
        print "# |  %3d: %10.5f %s" %  (i+1, freq, frequency_unit)
    print "#"

    if (control.phonon["band"] != []):
        print "# post-processing band structure"
        post_process_band(phonopy_obj, control.phonon["band"], frequency_unit_factor, 
                          is_eigenvectors=options.eigenvectors, write_yaml=options.yaml, 
                          do_matplotlib=do_matplotlib, lookup_labels=options.greek_labels)
        print "#"
    
    if (control.phonon["dos"] != {}) or (control.phonon["free_energy"] != {}):
        meshes = []
        if "qdensity" in control.phonon["dos"]:
            qdensity_dos = control.phonon["dos"]["qdensity"]
            if len(qdensity_dos) >= 3:
                mesh_dos = qdensity_dos[0:3] 
            else: 
                mesh_dos = [ qdensity_dos[0] ] * 3
            meshes.append(mesh_dos)
        if "qdensity" in control.phonon["free_energy"]:
            qdensity_free_energy = control.phonon["free_energy"]["qdensity"]
            if len(qdensity_free_energy) >= 3:
                mesh_free_energy = qdensity_free_energy[0:3] 
            else: 
                mesh_free_energy = [ qdensity_free_energy[0] ] * 3
            meshes.append(mesh_free_energy)
        mesh = sorted(meshes, reverse=True)[0]
        print "# creating q-points grid " + str(mesh)
        print "#"
        mesh_shift = [0,0,0]
        mesh_time_reversal = options.symmetry
        mesh_symmetry = options.symmetry
        mesh_eigenvectors = options.eigenvectors
        mesh_obj = Mesh(phonopy_obj.dynamical_matrix,
                        mesh, shift=mesh_shift,
                        is_time_reversal=mesh_time_reversal,
                        is_mesh_symmetry=mesh_symmetry,
                        is_eigenvectors=mesh_eigenvectors,
                        factor=frequency_unit_factor)
        if options.yaml:
            mesh_obj.write_yaml()
                        
        if (len(control.phonon["dos"]) > 0):
            print "# post-processing density of states"
            post_process_dos(phonopy_obj, mesh_obj, control.phonon["dos"], do_matplotlib=do_matplotlib)
            print "#"
        if (len(control.phonon["free_energy"]) > 0):
            print "# post-processing free energy"
            post_process_free_energy(phonopy_obj, mesh_obj, control.phonon["free_energy"], 
                                     frequency_unit_to_THz, write_yaml=options.yaml, do_matplotlib=do_matplotlib)
            print "#"

    if (control.phonon["animation"] != []):
        print "# post-processing animation output"
        post_process_animation(phonopy_obj, control.phonon["animation"], frequency_unit_factor)
        print "#"

    
    if (control.phonon["modulations"] != {}):
        print "# post-processing modulations output"
        post_process_write_modulations(phonopy_obj,control.phonon["modulations"])
        print "#"

    if options.EigVecGamma:
        write_eigenfrequencies_and_eigenvectors_at_gamma(phonopy_obj)
    print "# finished successfully"
    




# Copyright (C) 2011 Atsushi Togo
# All rights reserved.
#
# This file is part of phonopy.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
# * Neither the name of the phonopy project nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import sys
import os
import numpy as np
from phonopy import Phonopy, PhonopyGruneisen, PhonopyQHA, __version__
import phonopy.file_IO as file_IO
from phonopy.cui.settings import PhonopyConfParser
from phonopy.cui.show_symmetry import check_symmetry
from phonopy.cui.phonopy_argparse import get_parser
from phonopy.units import (Wien2kToTHz, AbinitToTHz, PwscfToTHz, ElkToTHz,
                           SiestaToTHz, VaspToTHz, Hartree, Bohr)
from phonopy.structure.cells import print_cell, determinant
from phonopy.structure.atoms import atom_data, symbol_map
from phonopy.interface import create_FORCE_SETS, read_crystal_structure
from phonopy.interface.phonopy_yaml import PhonopyYaml

phonopy_version = __version__

# AA is created at http://www.network-science.de/ascii/.
def print_phononpy():
    print("""        _
  _ __ | |__   ___  _ __   ___   _ __  _   _
 | '_ \| '_ \ / _ \| '_ \ / _ \ | '_ \| | | |
 | |_) | | | | (_) | | | | (_) || |_) | |_| |
 | .__/|_| |_|\___/|_| |_|\___(_) .__/ \__, |
 |_|                            |_|    |___/""")

def print_version(version):
    print(('%s' % version).rjust(44))
    print('')

def print_end():
    print("""                 _
   ___ _ __   __| |
  / _ \ '_ \ / _` |
 |  __/ | | | (_| |
  \___|_| |_|\__,_|
""")

def print_error():
    print("""  ___ _ __ _ __ ___  _ __
 / _ \ '__| '__/ _ \| '__|
|  __/ |  | | | (_) | |
 \___|_|  |_|  \___/|_|
""")

def print_attention(attention_text):
    print("*" * 67)
    print(attention_text)
    print("*" * 67)
    print('')

def print_error_message(message):
    print('')
    print(message)

def file_exists(filename, log_level):
    if os.path.exists(filename):
        return True
    else:
        error_text = "%s not found." % filename
        print_error_message(error_text)
        if log_level > 0:
            print_error()
        sys.exit(1)

def finalize_phonopy(log_level,
                     phonopy_conf,
                     phonon,
                     interface_mode,
                     filename="phonopy.yaml"):
    if log_level > 0:
        phpy_yaml = PhonopyYaml(configuration=phonopy_conf.get_configures(),
                                calculator=interface_mode)
        phpy_yaml.set_phonon_info(phonon)
        with open(filename, 'w') as w:
            w.write(str(phpy_yaml))
        print_end()
    sys.exit(0)

def print_cells(phonon, unitcell_filename):
    print("Crsytal structure is read from \'%s\'." % unitcell_filename)
    supercell = phonon.get_supercell()
    unitcell = phonon.get_unitcell()
    primitive = phonon.get_primitive()
    p2p_map = primitive.get_primitive_to_primitive_map()
    mapping = np.array(
        [p2p_map[x] for x in primitive.get_supercell_to_primitive_map()],
        dtype='intc')
    s_indep_atoms = phonon.get_symmetry().get_independent_atoms()
    p_indep_atoms = mapping[s_indep_atoms]
    if unitcell.get_number_of_atoms() == primitive.get_number_of_atoms():
        print("-" * 32 + " unit cell " + "-" * 33)
        print_cell(primitive, stars=p_indep_atoms)
    else:
        u2s_map = supercell.get_unitcell_to_supercell_map()
        print("-" * 30 + " primitive cell " + "-" * 30)
        print_cell(primitive, stars=p_indep_atoms)
        print("-" * 32 + " unit cell " +  "-" * 33) # 32 + 11 + 33 = 76
        u2u_map = supercell.get_unitcell_to_unitcell_map()
        u_indep_atoms = [u2u_map[x] for x in s_indep_atoms]
        print_cell(unitcell, mapping=mapping[u2s_map], stars=u_indep_atoms)
    print("-" * 32 + " super cell " + "-" * 32)
    print_cell(supercell, mapping=mapping, stars=s_indep_atoms)
    print("-" * 76)

def print_settings(settings):
    run_mode = settings.get_run_mode()
    if run_mode == 'band':
        print("Band structure mode")
    if run_mode == 'mesh':
        print("Mesh sampling mode")
    if run_mode == 'band_mesh':
        print("Band structure and mesh sampling mode")
    if run_mode == 'anime':
        print("Animation mode")
    if run_mode == 'modulation':
        print("Modulation mode")
    if run_mode == 'irreps':
        print("Ir-representation mode")
    if run_mode == 'qpoints':
        if settings.get_write_dynamical_matrices():
            print("QPOINTS mode (dynamical matrices written out)")
        else:
            print("QPOINTS mode")
    if (run_mode == 'band' or
        run_mode == 'mesh' or
        run_mode == 'qpoints') and settings.get_is_group_velocity():
        gv_delta_q = settings.get_group_velocity_delta_q()
        if gv_delta_q is not None:
            print("  With group velocity calculation (dq=%3.1e)" % gv_delta_q)
        else:
            print('')
    if run_mode == 'displacements':
        print("Creating displacements")
        if not settings.get_is_plusminus_displacement() == 'auto':
            if settings.get_is_plusminus_displacement():
                print("  Plus Minus displacement: full plus minus directions")
            else:
                print("  Plus Minus displacement: only one direction")
        if not settings.get_is_diagonal_displacement():
            print("  Diagonal displacement: off")

    print("Settings:")
    if settings.get_is_nac():
        print("  Non-analytical term correction: on")
    if settings.get_fc_spg_symmetry():
        print("  Enforce space group symmetry to force constants: on")
    if settings.get_fc_symmetry_iteration() > 0:
        print("  Force constants symmetrization: %d times" %
              settings.get_fc_symmetry_iteration())
    if settings.get_lapack_solver():
        print("  Use Lapack solver via Lapacke: on")
    if run_mode == 'mesh' or run_mode == 'band_mesh':
        print("  Sampling mesh: %s" % np.array(settings.get_mesh()[0]))

    if (np.diag(np.diag(settings.get_supercell_matrix())) \
            - settings.get_supercell_matrix()).any():
        print("  Supercell matrix:")
        for v in settings.get_supercell_matrix():
            print("    %s" % v)
    else:
        print("  Supercell: %s" % np.diag(settings.get_supercell_matrix()))
    if settings.get_primitive_matrix() is not None:
        print("  Primitive axis:")
        for v in settings.get_primitive_matrix():
            print("    %s" % v)

###########################
# Primitive option parser #
###########################

parser = get_parser()
(options, args) = parser.parse_args()
option_list = parser.option_list

# Set log level
log_level = 1
if options.verbose:
    log_level = 2
if options.quiet or options.is_check_symmetry:
    log_level = 0
if options.loglevel is not None:
    log_level = options.loglevel

#
# Phonopy interface mode
#
# Physical units: energy,  distance,  atomic mass, force
# vasp          : eV,      Angstrom,  AMU,         eV/Angstrom
# wien2k        : Ry,      au(=borh), AMU,         mRy/au
# abinit        : hartree, au,        AMU,         eV/Angstrom
# elk           : hartree, au,        AMU,         hartree/au
# pwscf         : Ry,      au,        AMU,         Ry/au
# siesta        : eV,      au,        AMU,         eV/Angstroem
#
if options.wien2k_mode:
    interface_mode = 'wien2k'
    from phonopy.interface.wien2k import write_supercells_with_displacements
    factor = Wien2kToTHz
    nac_factor = 2000.0
    distance_to_A = Bohr
elif options.abinit_mode:
    interface_mode = 'abinit'
    from phonopy.interface.abinit import write_supercells_with_displacements
    factor = AbinitToTHz
    nac_factor = Hartree / Bohr
    distance_to_A = Bohr
elif options.pwscf_mode:
    interface_mode = 'pwscf'
    from phonopy.interface.pwscf import write_supercells_with_displacements
    factor = PwscfToTHz
    nac_factor = 2.0
    distance_to_A = Bohr
elif options.elk_mode:
    interface_mode = 'elk'
    from phonopy.interface.elk import write_supercells_with_displacements
    factor = ElkToTHz
    nac_factor = 1.0
    distance_to_A = Bohr
elif options.siesta_mode:
    interface_mode = 'siesta'
    from phonopy.interface.siesta import write_supercells_with_displacements
    factor = SiestaToTHz
    nac_factor = Hartree / Bohr
    distance_to_A = Bohr
else:
    if options.vasp_mode:
        interface_mode = 'vasp'
        distance_to_A = 1.0
    else:
        interface_mode = None
        distance_to_A = 1.0
    from phonopy.interface.vasp import write_supercells_with_displacements
    from phonopy.interface.vasp import create_FORCE_CONSTANTS
    factor = VaspToTHz
    nac_factor = Hartree * Bohr

if options.is_graph_save:
    import matplotlib
    matplotlib.use('Agg')

################
# Phonopy logo #
################

if log_level > 0:
    print_phononpy()
    print_version(phonopy_version)

###########################
# Integrated helper tools #
###########################

# Create FORCE_SETS (-f or --force_sets)
if options.force_sets_mode or options.force_sets_zero_mode:
    file_exists('disp.yaml', log_level)
    for filename in args:
        file_exists(filename, log_level)
    error_num = create_FORCE_SETS(
        interface_mode,
        args,
        options.symprec,
        is_wien2k_p1=options.is_wien2k_p1,
        force_sets_zero_mode=options.force_sets_zero_mode,
        log_level=log_level)
    if log_level > 0:
        print_end()
    sys.exit(error_num)

# Create FORCE_CONSTANTS (--fc or --force_constants)
if options.force_constants_mode:
    if len(args) > 0:
        file_exists(args[0], log_level)
        error_num = create_FORCE_CONSTANTS(args[0],
                                           options,
                                           log_level)
    else:
        print_error_message("Please specify vasprun.xml.")
        error_num = 1

    if log_level > 0:
        print_end()
    sys.exit(error_num)

#########################
# Read phonopy settings #
#########################
if len(args) > 0:
    if file_exists(args[0], log_level):
        phonopy_conf = PhonopyConfParser(filename=args[0],
                                         options=options,
                                         option_list=option_list)
        settings = phonopy_conf.get_settings()
else:
    phonopy_conf = PhonopyConfParser(options=options,
                                     option_list=option_list)
    settings = phonopy_conf.get_settings()

##################
# Read unit cell #
##################
unitcell, optional_structure_file_information = read_crystal_structure(
    filename=settings.get_cell_filename(),
    interface_mode=interface_mode,
    chemical_symbols=settings.get_chemical_symbols(),
    yaml_mode=settings.get_yaml_mode())
unitcell_filename = optional_structure_file_information[0]

if unitcell is None:
    print_error_message("Crystal structure file of %s could not be found." %
                        unitcell_filename)
    if log_level > 0:
        print_error()
    sys.exit(1)

# Check unit cell
if np.linalg.det(unitcell.get_cell()) < 0.0:
    print_error_message("Lattice vectors have to follow the right-hand rule.")
    if log_level > 0:
        print_end()
    sys.exit(1)

# Set magnetic moments
magmoms = settings.get_magnetic_moments()
if magmoms is not None:
    if len(magmoms) == unitcell.get_number_of_atoms():
        unitcell.set_magnetic_moments(magmoms)
    else:
        error_text = "Invalid MAGMOM setting"
        print_error_message(error_text)
        if log_level > 0:
            print_end()
        sys.exit(1)

################################################
# Check crystal symmetry and exit (--symmetry) #
################################################
if options.is_check_symmetry:
    check_symmetry(unitcell,
                   primitive_axis=settings.get_primitive_matrix(),
                   symprec=options.symprec,
                   distance_to_A=distance_to_A,
                   phonopy_version=phonopy_version)
    if log_level > 0:
        print_end()
    sys.exit(0)

##########################
# Phonopy initialization #
##########################
run_mode = settings.get_run_mode()

if settings.get_supercell_matrix() is None:
    print_error_message("Supercell matrix (DIM or --dim) is not found.")
    if log_level > 0:
        print_end()
    sys.exit(1)

# Prepare phonopy object
if run_mode == 'displacements':
    phonon = Phonopy(unitcell,
                     settings.get_supercell_matrix(),
                     symprec=options.symprec,
                     is_symmetry=settings.get_is_symmetry(),
                     log_level=log_level)
else: # Read FORCE_SETS, FORCE_CONSTANTS, or force_constants.hdf5
    num_atom = unitcell.get_number_of_atoms()
    num_satom = determinant(settings.get_supercell_matrix()) * num_atom
    if settings.get_is_force_constants() == 'read':
        if settings.get_is_hdf5():
            try:
                import h5py
            except ImportError:
                print_error_message("You need to install python-h5py.")
                if log_level > 0:
                    print_end()
                sys.exit(1)

            if file_exists("force_constants.hdf5", log_level):
                fc = file_IO.read_force_constants_hdf5("force_constants.hdf5")
                fc_filename = "force_constants.hdf5"

        elif file_exists("FORCE_CONSTANTS", log_level):
            fc = file_IO.parse_FORCE_CONSTANTS(filename="FORCE_CONSTANTS")
            fc_filename = "FORCE_CONSTANTS"

        if log_level > 0:
            print("Force constants are read from %s." % fc_filename)

        if fc.shape[0] != num_satom:
            error_text = ("Number of atoms in supercell is not consistent with "
                          "the matrix shape of\nforce constants read from ")
            if settings.get_is_hdf5():
                error_text += "force_constants.hdf5.\n"
            else:
                error_text += "FORCE_CONSTANTS.\n"
            error_text += ("Please carefully check DIM, FORCE_CONSTANTS, "
                           "and %s.") % unitcell_filename
            print_error_message(error_text)
            if log_level > 0:
                print_end()
            sys.exit(1)

    elif file_exists("FORCE_SETS", log_level):
        force_sets = file_IO.parse_FORCE_SETS()
        if force_sets['natom'] != num_satom:
            error_text = "Number of atoms in supercell is not consistent with "
            error_text += "the data in FORCE_SETS.\n"
            error_text += ("Please carefully check DIM, FORCE_SETS,"
                           " and %s") % unitcell_filename
            print_error_message(error_text)
            if log_level > 0:
                print_end()
            sys.exit(1)

    # Overwrite frequency unit conversion factor
    if settings.get_frequency_conversion_factor() is not None:
        factor = settings.get_frequency_conversion_factor()

    phonon = Phonopy(unitcell,
                     settings.get_supercell_matrix(),
                     primitive_matrix=settings.get_primitive_matrix(),
                     factor=factor,
                     dynamical_matrix_decimals=settings.get_dm_decimals(),
                     force_constants_decimals=settings.get_fc_decimals(),
                     symprec=options.symprec,
                     is_symmetry=settings.get_is_symmetry(),
                     use_lapack_solver=settings.get_lapack_solver(),
                     log_level=log_level)

supercell = phonon.get_supercell()
primitive = phonon.get_primitive()

# Set atomic masses of primitive cell
if settings.get_masses() is not None:
    phonon.set_masses(settings.get_masses())

# Show logs
if log_level > 0:
    print("Python version %d.%d.%d" % sys.version_info[:3])
    import phonopy.structure.spglib as spglib
    print("Spglib version %d.%d.%d" % spglib.get_version())
    if interface_mode:
        print("Calculator interface: %s" % interface_mode)
    print_settings(settings)
    if magmoms is None:
        print("Spacegroup: %s" %
              phonon.get_symmetry().get_international_table())

# Print cells
if log_level > 1:
    print_cells(phonon, unitcell_filename)

#################################
# Create displacements and exit #
#################################
if run_mode == 'displacements':
    if settings.get_displacement_distance() is None:
        if (interface_mode == 'wien2k' or
            interface_mode == 'abinit' or
            interface_mode == 'elk' or
            interface_mode == 'pwscf' or
            interface_mode == 'siesta'):
            displacement_distance = 0.02
        else: # default or vasp
            displacement_distance = 0.01
    else:
        displacement_distance = settings.get_displacement_distance()

    phonon.generate_displacements(
        distance=displacement_distance,
        is_plusminus=settings.get_is_plusminus_displacement(),
        is_diagonal=settings.get_is_diagonal_displacement(),
        is_trigonal=settings.get_is_trigonal_displacement())
    displacements = phonon.get_displacements()
    directions = phonon.get_displacement_directions()
    file_IO.write_disp_yaml(displacements,
                            supercell,
                            directions=directions)

    # Write supercells with displacements
    cells_with_disps = phonon.get_supercells_with_displacements()

    if interface_mode == 'wien2k':
        npts, r0s, rmts = optional_structure_file_information[1:4]
        write_supercells_with_displacements(
            supercell,
            cells_with_disps,
            npts,
            r0s,
            rmts,
            settings.get_supercell_matrix(),
            filename=unitcell_filename)
    elif interface_mode == 'abinit':
        write_supercells_with_displacements(supercell, cells_with_disps)
    elif (interface_mode == 'pwscf' or
          interface_mode == 'elk' or
          interface_mode == 'siesta'):
        write_supercells_with_displacements(
            supercell,
            cells_with_disps,
            optional_structure_file_information[1])
    else: # default or vasp
        write_supercells_with_displacements(supercell, cells_with_disps)

    if log_level > 0:
        print('')
        print("disp.yaml and supercells have been created.")

    finalize_phonopy(log_level,
                     phonopy_conf,
                     phonon,
                     interface_mode,
                     filename="phonopy_disp.yaml")

########################################
# Preparations for phonon calculations #
########################################
if settings.get_is_force_constants() == 'read':
    phonon.set_force_constants(fc)
elif os.path.exists("FORCE_SETS"):
    phonon.set_displacement_dataset(force_sets)
    if log_level > 0:
        print("Computing force constants...")

    if (settings.get_is_force_constants() == "write" or
        settings.get_fc_symmetry_iteration() > 0 or
        settings.get_fc_spg_symmetry()):
        # Need to calculate full force constant tensors
        phonon.produce_force_constants(
            computation_algorithm=settings.get_fc_computation_algorithm())
    else: # Only force constants between atoms in primitive cell and in supercell
        phonon.produce_force_constants(
            calculate_full_force_constants=False,
            computation_algorithm=settings.get_fc_computation_algorithm())

# Non-analytical term correction (LO-TO splitting)
if settings.get_is_nac():
    if file_exists("BORN", log_level):
        nac_params = file_IO.get_born_parameters(
            open("BORN"),
            primitive,
            phonon.get_primitive_symmetry())
        if not nac_params:
            error_text = "BORN file could not be read correctly."
            print_error_message(error_text)
            if log_level > 0:
                print_end()
            sys.exit(1)
        if nac_params['factor'] == None:
            nac_params['factor'] = nac_factor
        phonon.set_nac_params(nac_params=nac_params)

        if log_level > 1:
            print("-" * 27 + " Dielectric constant " + "-" * 28)
            for v in nac_params['dielectric']:
                print("         %12.7f %12.7f %12.7f" % tuple(v))
            print("-" * 26 + " Born effective charges " + "-" * 26)
            symbols = primitive.get_chemical_symbols()
            for i, (z, s) in enumerate(zip(nac_params['born'], symbols)):
                for j, v in enumerate(z):
                    if j == 0:
                        text = "%5d %-2s" % (i + 1, s)
                    else:
                        text = "        "
                    print("%s %12.7f %12.7f %12.7f" % ((text,) + tuple(v)))
            print("-" * 76)

# Impose cutoff radius on force constants
cutoff_radius = settings.get_cutoff_radius()
if cutoff_radius:
    phonon.set_force_constants_zero_with_radius(cutoff_radius)

# Enforce space group symmetry to force constants
if settings.get_fc_spg_symmetry():
    if log_level > 0:
        print('')
        print("Force constants are symmetrized by space group operations.")
        print("This may take some time...")
    phonon.symmetrize_force_constants_by_space_group()
    file_IO.write_FORCE_CONSTANTS(phonon.get_force_constants(),
                                  filename='FORCE_CONSTANTS_SPG')
    if log_level > 0:
        print("Symmetrized force constants are written into "
              "FORCE_CONSTANTS_SPG.")

# Imporse translational invariance and index permulation symmetry to
# force constants
if settings.get_fc_symmetry_iteration() > 0:
    phonon.symmetrize_force_constants(settings.get_fc_symmetry_iteration())

# Write FORCE_CONSTANTS
if settings.get_is_force_constants() == "write":
    if settings.get_is_hdf5():
        file_IO.write_force_constants_to_hdf5(phonon.get_force_constants())
        if log_level > 0:
            print("Force constants are written into force_constants.hdf5.")
    else:
        file_IO.write_FORCE_CONSTANTS(phonon.get_force_constants())
        if log_level > 0:
            print("Force constants are written into FORCE_CONSTANTS.")

# Show the rotational invariance condition (just show!)
if settings.get_is_rotational_invariance():
    phonon.get_rotational_condition_of_fc()

# Atomic species without mass case
symbols_with_no_mass = []
if primitive.get_masses() is None:
    for s in primitive.get_chemical_symbols():
        if (atom_data[symbol_map[s]][3] is None and
            s not in symbols_with_no_mass):
            symbols_with_no_mass.append(s)
            print_error_message(
                "Atomic mass of \'%s\' is not implemented in phonopy." % s)
            print_error_message("MASS tag can be used to set atomic masses.")

if len(symbols_with_no_mass) > 0:
    if log_level > 0:
        print_end()
    sys.exit(1)

# Group velocity
if settings.get_is_group_velocity():
    phonon.set_group_velocity(q_length=settings.get_group_velocity_delta_q())

#######################
# Phonon calculations #
#######################

# QPOINTS mode
if run_mode == 'qpoints':
    if settings.get_qpoints():
        q_points = settings.get_qpoints()
        if log_level > 0:
            print("Q-points that will be calculated at:")
            for q in q_points:
                print("    %s" % q)
    else:
        q_points = file_IO.parse_QPOINTS()
        if log_level > 0:
            print("Frequencies at q-points given by QPOINTS:")
    phonon.set_qpoints_phonon(
        q_points,
        nac_q_direction=settings.get_nac_q_direction(),
        is_eigenvectors=settings.get_is_eigenvectors(),
        write_dynamical_matrices=settings.get_write_dynamical_matrices())

    if settings.get_is_hdf5():
        phonon.write_hdf5_qpoints_phonon()
    else:
        phonon.write_yaml_qpoints_phonon()

# Band structure and mesh sampling
elif run_mode == 'band' or run_mode == 'mesh' or run_mode == 'band_mesh':
    if run_mode == 'band' or run_mode == 'band_mesh':
        bands = settings.get_bands()
        if log_level > 0:
            print("Reciprocal space paths in reduced coordinates:")
            for band in bands:
                print("[%5.2f %5.2f %5.2f] --> [%5.2f %5.2f %5.2f]" %
                      (tuple(band[0]) + tuple(band[-1])))

        phonon.set_band_structure(
            bands,
            is_eigenvectors=settings.get_is_eigenvectors(),
            is_band_connection=settings.get_is_band_connection())
        if interface_mode is None:
            comment = None
        else:
            comment = {'calculator': interface_mode}
        phonon.write_yaml_band_structure(labels=settings.get_band_labels(),
                                         comment=comment)

        if options.is_graph_plot and run_mode != 'band_mesh':
            plot = phonon.plot_band_structure(labels=settings.get_band_labels())
            if options.is_graph_save:
                plot.savefig('band.pdf')
            else:
                plot.show()

    if run_mode == 'mesh' or run_mode == 'band_mesh':
        (mesh,
         mesh_shift,
         t_symmetry,
         q_symmetry,
         is_gamma_center) =  settings.get_mesh()
        phonon.set_mesh(mesh,
                        mesh_shift,
                        is_time_reversal=t_symmetry,
                        is_mesh_symmetry=q_symmetry,
                        is_eigenvectors=settings.get_is_eigenvectors(),
                        is_gamma_center=settings.get_is_gamma_center())
        weights = phonon.get_mesh()[1]
        if log_level > 0:
            if q_symmetry:
                print("Number of irreducible q-points on sampling mesh: "
                      "%d/%d" % (weights.shape[0], np.prod(mesh)))
            else:
                print("Number of q-points on sampling mesh: %d" %
                      weights.shape[0])

        if settings.get_write_mesh():
            if settings.get_is_hdf5():
                phonon.write_hdf5_mesh()
            else:
                phonon.write_yaml_mesh()

        # Thermal property
        if settings.get_is_thermal_properties():

            if log_level > 0:
                if settings.get_is_projected_thermal_properties():
                    print("Calculating projected thermal properties...")
                else:
                    print("Calculating thermal properties...")
            tprop_range = settings.get_thermal_property_range()
            tstep = tprop_range['step']
            tmax = tprop_range['max']
            tmin = tprop_range['min']
            phonon.set_thermal_properties(
                tstep,
                tmax,
                tmin,
                is_projection=settings.get_is_projected_thermal_properties(),
                band_indices=settings.get_band_indices(),
                cutoff_frequency=settings.get_cutoff_frequency())
            phonon.write_yaml_thermal_properties()

            if log_level > 0:
                print("#%11s %15s%15s%15s%15s" % ('T [K]',
                                                  'F [kJ/mol]',
                                                  'S [J/K/mol]',
                                                  'C_v [J/K/mol]',
                                                  'E [kJ/mol]'))
                (temps,
                 fe,
                 entropy,
                 heat_capacity) = phonon.get_thermal_properties()
                for T, F, S, CV in zip(temps, fe, entropy, heat_capacity):
                    print(("%12.3f " + "%15.7f" * 4) %
                          (T, F, S, CV, F + T * S / 1000))

            if options.is_graph_plot:
                plot = phonon.plot_thermal_properties()
                if options.is_graph_save:
                    plot.savefig('thermal_properties.pdf')
                else:
                    plot.show()

        # Thermal displacements
        elif settings.get_is_thermal_displacements():
            p_direction = settings.get_projection_direction()
            if log_level > 0 and p_direction is not None:
                c_direction = np.dot(p_direction, primitive.get_cell())
                c_direction /= np.linalg.norm(c_direction)
                print("Projection direction: [%6.3f %6.3f %6.3f] "
                      "(fractional)" % tuple(p_direction))
                print("                      [%6.3f %6.3f %6.3f] "
                      "(Cartesian)" % tuple(c_direction))
            if log_level > 0:
                print("Calculating thermal displacements...")
            tprop_range = settings.get_thermal_property_range()
            tstep = tprop_range['step']
            tmax = tprop_range['max']
            tmin = tprop_range['min']
            phonon.set_thermal_displacements(
                tstep,
                tmax,
                tmin,
                direction=p_direction,
                cutoff_frequency=settings.get_cutoff_frequency())
            phonon.write_yaml_thermal_displacements()

            if options.is_graph_plot:
                plot = phonon.plot_thermal_displacements(options.is_legend)
                if options.is_graph_save:
                    plot.savefig('thermal_displacement.pdf')
                else:
                    plot.show()

        # Thermal displacement matrices
        elif settings.get_is_thermal_displacement_matrices():
            if log_level > 0:
                print("Calculating thermal displacement matrices...")
            tprop_range = settings.get_thermal_property_range()
            t_step = tprop_range['step']
            t_max = tprop_range['max']
            t_min = tprop_range['min']
            t_cif = settings.get_thermal_displacement_matrix_temperature()
            phonon.set_thermal_displacement_matrices(
                t_step=t_step,
                t_max=t_max,
                t_min=t_min,
                cutoff_frequency=settings.get_cutoff_frequency(),
                t_cif=t_cif)
            phonon.write_yaml_thermal_displacement_matrices()
            if t_cif is not None:
                phonon.write_thermal_displacement_matrix_to_cif(0)

        # Thermal distances
        elif settings.get_is_thermal_distances():
            if log_level > 0:
                print("Calculating thermal distances...")
            tprop_range = settings.get_thermal_property_range()
            tstep = tprop_range['step']
            tmax = tprop_range['max']
            tmin = tprop_range['min']
            phonon.set_thermal_distances(
                settings.get_thermal_atom_pairs(),
                tstep,
                tmax,
                tmin,
                cutoff_frequency=settings.get_cutoff_frequency())
            phonon.write_yaml_thermal_distances()

        # Partial DOS
        elif settings.get_pdos_indices() is not None:
            p_direction = settings.get_projection_direction()
            if (log_level > 0 and
                p_direction is not None and
                not settings.get_xyz_projection()):
                c_direction = np.dot(p_direction, primitive.get_cell())
                c_direction /= np.linalg.norm(c_direction)
                print("Projection direction: [%6.3f %6.3f %6.3f] "
                      "(fractional)" % tuple(p_direction))
                print("                      [%6.3f %6.3f %6.3f] "
                      "(Cartesian)" % tuple(c_direction))
            if log_level > 0:
                print("Calculating partial PDOS...")
            dos_range = settings.get_dos_range()
            phonon.set_partial_DOS(
                sigma=settings.get_sigma(),
                freq_min=dos_range['min'],
                freq_max=dos_range['max'],
                freq_pitch=dos_range['step'],
                tetrahedron_method=settings.get_is_tetrahedron_method(),
                direction=p_direction,
                xyz_projection=settings.get_xyz_projection())
            phonon.write_partial_DOS()

            if options.is_graph_plot:
                pdos_indices = settings.get_pdos_indices()
                if not pdos_indices:
                    num_atom = primitive.get_number_of_atoms()
                    pdos_indices = [np.arange(num_atom) * 3 + i
                                    for i in range(3)]

            if options.is_graph_plot and run_mode != 'band_mesh':
                plot = phonon.plot_partial_DOS(
                    pdos_indices=pdos_indices,
                    legend=([np.array(x) + 1 for x in pdos_indices]))
                if options.is_graph_save:
                    plot.savefig('partial_dos.pdf')
                else:
                    plot.show()

        # Total DOS
        elif (options.is_graph_plot or settings.get_is_dos_mode()):
            dos_range = settings.get_dos_range()

            phonon.set_total_DOS(
                sigma=settings.get_sigma(),
                freq_min=dos_range['min'],
                freq_max=dos_range['max'],
                freq_pitch=dos_range['step'],
                tetrahedron_method=settings.get_is_tetrahedron_method())

            if log_level > 0:
                print("Calculating DOS...")

            if settings.get_fits_Debye_model():
                phonon.set_Debye_frequency()
                if log_level > 0:
                    debye_freq = phonon.get_Debye_frequency()
                    print("Debye frequency: %10.5f" % debye_freq)
            phonon.write_total_DOS()

            if options.is_graph_plot and run_mode != 'band_mesh':
                plot = phonon.plot_total_DOS()
                if options.is_graph_save:
                    plot.savefig('total_dos.pdf')
                else:
                    plot.show()

        elif settings.get_is_moment():
            dos_range = settings.get_dos_range()
            freq_min = dos_range['min']
            freq_max = dos_range['max']
            if log_level > 0:
                text = "Calculating moment of phonon states distribution"
                if freq_min is None and freq_max is None:
                    text += "..."
                elif freq_min is None and freq_max is not None:
                    text += "\nbelow frequency %.3f..." % freq_max
                elif freq_min is not None and freq_max is None:
                    text += "\nabove frequency %.3f..." % freq_min
                elif freq_min is not None and freq_max is not None:
                    text += ("\nbetween frequencies %.3f and %.3f..." %
                             (freq_min, freq_max))
            print(text)
            print('')
            print("Order|   Total   |   Projected to atoms")
            if settings.get_moment_order() is not None:
                phonon.set_moment(order=settings.get_moment_order(),
                                  freq_min=freq_min,
                                  freq_max=freq_max,
                                  is_projection=False)
                total_moment = phonon.get_moment()
                phonon.set_moment(order=settings.get_moment_order(),
                                  freq_min=freq_min,
                                  freq_max=freq_max,
                                  is_projection=True)
                text = " %3d |%10.5f | " % (settings.get_moment_order(),
                                            total_moment)
                for m in phonon.get_moment():
                    text += "%10.5f " % m
                print(text)
            else:
                for i in range(3):
                    phonon.set_moment(order=i,
                                      freq_min=freq_min,
                                      freq_max=freq_max,
                                      is_projection=False)
                    total_moment = phonon.get_moment()
                    phonon.set_moment(order=i,
                                      freq_min=freq_min,
                                      freq_max=freq_max,
                                      is_projection=True)
                    text = " %3d |%10.5f | " % (i, total_moment)
                    for m in phonon.get_moment():
                        text += "%10.5f " % m
                    print(text)

    if (run_mode == 'band_mesh' and
        options.is_graph_plot and
        not settings.get_is_thermal_properties() and
        not settings.get_is_thermal_displacements() and
        not settings.get_is_thermal_displacement_matrices() and
        not settings.get_is_thermal_distances()):
        if settings.get_pdos_indices() is not None:
            plot = phonon.plot_band_structure_and_dos(
                pdos_indices=pdos_indices,
                labels=settings.get_band_labels())
        else:
            plot = phonon.plot_band_structure_and_dos(
                labels=settings.get_band_labels())
        if options.is_graph_save:
            plot.savefig('band_dos.pdf')
        else:
            plot.show()


# Animation
elif run_mode == 'anime':
    anime_type = settings.get_anime_type()
    if anime_type == "v_sim":
        q_point = settings.get_anime_qpoint()
        amplitude = settings.get_anime_amplitude()
        phonon.write_animation(q_point=q_point,
                               anime_type='v_sim',
                               amplitude=amplitude)
        if log_level > 0:
            print("Animation type: v_sim")
            print("q-point: [%6.3f %6.3f %6.3f]" % tuple(q_point))
    else:
        amplitude = settings.get_anime_amplitude()
        band_index = settings.get_anime_band_index()
        division = settings.get_anime_division()
        shift = settings.get_anime_shift()
        phonon.write_animation(anime_type=anime_type,
                               band_index=band_index,
                               amplitude=amplitude,
                               num_div=division,
                               shift=shift)

        if log_level > 0:
            print("Animation type: %s" % anime_type)
            print("amplitude: %f" % amplitude)
            if anime_type != "jmol":
                print("band index: %d" % band_index)
                print("Number of images: %d" % division)

# Modulation
elif run_mode == 'modulation':
    mod_setting = settings.get_modulation()
    phonon_modes = mod_setting['modulations']
    dimension = mod_setting['dimension']
    if 'delta_q' in mod_setting:
        delta_q = mod_setting['delta_q']
    else:
        delta_q = None
    derivative_order = mod_setting['order']

    phonon.set_modulations(dimension,
                           phonon_modes,
                           delta_q=delta_q,
                           derivative_order=derivative_order,
                           nac_q_direction=settings.get_nac_q_direction())
    phonon.write_modulations()
    phonon.write_yaml_modulations()

# Ir-representation
elif run_mode == 'irreps':
    if phonon.set_irreps(settings.get_irreps_q_point(),
                         is_little_cogroup=settings.get_is_little_cogroup(),
                         nac_q_direction=settings.get_nac_q_direction(),
                         degeneracy_tolerance=settings.get_irreps_tolerance()):
        show_irreps = settings.get_show_irreps()
        phonon.show_irreps(show_irreps)
        phonon.write_yaml_irreps(show_irreps)

else:
    print("-" * 76)
    print(" One of the following run modes may be specified for phonon "
          "calculations.")
    for mode in ['Mesh sampling (MP, --mesh)',
                 'Q-points (QPOINTS, --qpoints)',
                 'Band structure (BAND, --band)',
                 'Animation (ANIME, --anime)',
                 'Modulation (MODULATION, --modulation)',
                 'Characters of Irreps (IRREPS, --irreps)',
                 'Create displacements (-d)']:
        print(" - %s" % mode)
    print("-" * 76)

########################
# Phonopy finalization #
########################
finalize_phonopy(log_level, phonopy_conf, phonon, interface_mode)



# Copyright (C) 2012 Atsushi Togo
# All rights reserved.
#
# This file is part of phonopy.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
# * Neither the name of the phonopy project nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import numpy as np
from phonopy.units import Avogadro, EvTokJmol, EVAngstromToGPa
from phonopy.qha.eos import get_eos, fit_to_eos

class BulkModulus(object):
    def __init__(self,
                 volumes,
                 electronic_energies,
                 eos='vinet'):
        self._volumes = volumes
        self._electronic_energies = electronic_energies
        self._eos = get_eos(eos)
        (self._energy,
         self._bulk_modulus,
         self._b_prime,
         self._volume) = fit_to_eos(volumes,
                                    electronic_energies,
                                    self._eos)

    def get_bulk_modulus(self):
        return self._bulk_modulus

    def get_equilibrium_volume(self):
        return self._volume

    def get_b_prime(self):
        return self._b_prime

    def get_energy(self):
        return self._energy

    def get_parameters(self):
        return (self._energy,
                self._bulk_modulus,
                self._b_prime,
                self._volume)

    def get_eos(self):
        return self._eos

    def plot(self):
        import matplotlib.pyplot as plt
        ep = self.get_parameters()
        vols = self._volumes
        plt.plot(vols, self._electronic_energies, 'bo', markersize=4)
        volume_points = np.linspace(min(vols), max(vols), 201)
        plt.plot(volume_points, self._eos(volume_points, *ep), 'r-')
        return plt


class QHA(object):
    def __init__(self,
                 volumes, # Angstrom^3
                 electronic_energies, # eV
                 temperatures,
                 cv,        # J/K/mol
                 entropy,   # J/K/mol
                 fe_phonon, # kJ/mol
                 eos='vinet',
                 t_max=None):
        self._volumes = np.array(volumes)
        self._electronic_energies = np.array(electronic_energies)

        self._all_temperatures = np.array(temperatures)
        self._cv = np.array(cv)
        self._entropy = np.array(entropy)
        self._fe_phonon = np.array(fe_phonon) / EvTokJmol

        self._eos = get_eos(eos)
        self._t_max = t_max

        self._temperatures = []
        self._equiv_volumes = []
        self._equiv_energies = []
        self._equiv_bulk_modulus = []
        self._equiv_parameters = []
        self._free_energies = []

        self._thermal_expansions = None
        self._volume_expansions = None
        self._cp_numerical = None
        self._volume_entropy_parameters = None
        self._volume_cv_parameters = None
        self._volume_entropy = None
        self._volume_cv = None
        self._cp_polyfit = None
        self._dsdv = None
        self._gruneisen_parameters = None

    def run(self, verbose=False):
        if verbose:
            print(("#%11s" + "%14s" * 4) % ("T", "E_0", "B_0", "B'_0", "V_0"))

        max_t_index = self._get_max_t_index(self._all_temperatures)

        for i in range(max_t_index + 2):
            t = self._all_temperatures[i]
            fe = []
            for j, e in enumerate(self._electronic_energies):
                fe.append(e + self._fe_phonon[i][j])
            self._free_energies.append(fe)

            ee, eb, ebp, ev = fit_to_eos(self._volumes, fe, self._eos)
            if ee is None:
                continue
            else:
                ep = [ee, eb, ebp, ev]
                self._temperatures.append(t)
                self._equiv_volumes.append(ev)
                self._equiv_energies.append(ee)
                self._equiv_bulk_modulus.append(eb * EVAngstromToGPa)
                self._equiv_parameters.append(ep)

                if verbose:
                    print(("%14.6f" * 5) %
                          (t, ep[0], ep[1] * EVAngstromToGPa, ep[2], ep[3]))

        self._max_t_index = self._get_max_t_index(self._temperatures)
        self._set_volume_expansion()
        self._set_thermal_expansion() # len = len(t) - 1
        self._set_heat_capacity_P_numerical() # len = len(t) - 2
        self._set_heat_capacity_P_polyfit()
        self._set_gruneisen_parameter() # To be run after thermal expansion.

    def plot(self, thin_number=10, volume_temp_exp=None):
        import matplotlib.pyplot as plt
        plt.subplot(1, 3, 1)
        self._plot_helmholtz_volume(plt, thin_number=thin_number)
        plt.subplot(1, 3, 2)
        self._plot_volume_temperature(plt, exp_data=volume_temp_exp)
        plt.subplot(1, 3, 3)
        self._plot_thermal_expansion(plt)
        plt.subplots_adjust(wspace=0.35)
        return plt

    def get_eos(self):
        return self._eos

    def get_helmholtz_volume(self):
        return self._free_energies[:self._max_t_index]

    def plot_helmholtz_volume(self,
                              thin_number=10,
                              plt=None,
                              xlabel=r'Volume $(\AA^3)$',
                              ylabel='Free energy'):
        if plt is None:
            import matplotlib.pyplot as _plt
            self._plot_helmholtz_volume(_plt,
                                        thin_number=thin_number,
                                        xlabel=xlabel,
                                        ylabel=ylabel)
            return _plt
        else:
            self._plot_helmholtz_volume(plt,
                                        thin_number=thin_number,
                                        xlabel=xlabel,
                                        ylabel=ylabel)

    def plot_pdf_helmholtz_volume(self,
                                  thin_number=10,
                                  filename='helmholtz-volume.pdf'):
        import matplotlib.pyplot as plt
        plt.rcParams['backend'] = 'PDF'
        plt.rcParams['pdf.fonttype'] = 42
        plt.rcParams['font.family'] = 'serif'
        plt.rcParams['axes.labelsize'] = 18
        plt.rcParams['figure.subplot.left'] = 0.25
        plt.rcParams['figure.subplot.bottom'] = 0.15
        plt.rcParams['figure.figsize'] = 4, 6

        self._plot_helmholtz_volume(plt, thin_number=thin_number)
        plt.savefig(filename)
        plt.close()

    def write_helmholtz_volume(self, filename='helmholtz-volume.dat'):
        w = open(filename, 'w')
        for i in range(self._max_t_index):
            w.write("# Temperature: %f\n" % self._temperatures[i])
            w.write("# Parameters: %f %f %f %f\n" %
                    tuple(self._equiv_parameters[i]))
            for j, v in enumerate(self._volumes):
                w.write("%20.15f %25.15f\n" % (v, self._free_energies[i][j]))
            w.write("\n\n")
        w.close()

    def get_volume_temperature(self):
        return self._equiv_volumes[:self._max_t_index]

    def plot_volume_temperature(self, exp_data=None, plt=None):
        if plt is None:
            import matplotlib.pyplot as _plt
            self._plot_volume_temperature(_plt, exp_data=exp_data)
            return _plt
        else:
            self._plot_volume_temperature(plt, exp_data=exp_data)

    def plot_pdf_volume_temperature(self,
                                    exp_data=None,
                                    filename='volume-temperature.pdf'):
        import matplotlib.pyplot as plt
        plt.rcParams['backend'] = 'PDF'
        plt.rcParams['pdf.fonttype'] = 42
        plt.rcParams['font.family'] = 'serif'
        plt.rcParams['axes.labelsize'] = 18
        plt.rcParams['figure.subplot.left'] = 0.15
        plt.rcParams['figure.subplot.bottom'] = 0.15
        plt.rcParams['figure.figsize'] = 8, 6

        self._plot_volume_temperature(plt, exp_data=exp_data)
        plt.savefig(filename)
        plt.close()

    def write_volume_temperature(self, filename='volume-temperature.dat'):
        w = open(filename, 'w')
        for i in range(self._max_t_index):
            w.write("%25.15f %25.15f\n" % (self._temperatures[i],
                                           self._equiv_volumes[i]))
        w.close()

    def get_thermal_expansion(self):
        return self._thermal_expansions[:self._max_t_index]

    def plot_thermal_expansion(self, plt=None):
        if plt is None:
            import matplotlib.pyplot as _plt
            self._plot_thermal_expansion(_plt)
            return _plt
        else:
            self._plot_thermal_expansion(plt)

    def plot_pdf_thermal_expansion(self, filename='thermal_expansion.pdf'):
        import matplotlib.pyplot as plt
        plt.rcParams['backend'] = 'PDF'
        plt.rcParams['pdf.fonttype'] = 42
        plt.rcParams['font.family'] = 'serif'
        plt.rcParams['axes.labelsize'] = 18
        plt.rcParams['figure.subplot.left'] = 0.15
        plt.rcParams['figure.subplot.bottom'] = 0.15
        plt.rcParams['figure.figsize'] = 8, 6

        self._plot_thermal_expansion(plt)
        plt.savefig(filename)
        plt.close()

    def write_thermal_expansion(self, filename='thermal_expansion.dat'):
        w = open(filename, 'w')
        for i in range(self._max_t_index):
            w.write("%25.15f %25.15f\n" % (self._temperatures[i],
                                           self._thermal_expansions[i]))
        w.close()

    def get_volume_expansion(self):
        return self._volume_expansions[:self._max_t_index]

    def plot_volume_expansion(self, exp_data=None, symbol='o', _plt=None):
        if self._temperatures[self._max_t_index] > 300:
            if plt is None:
                import matplotlib.pyplot as _plt
                self._plot_volume_expansion(_plt,
                                            exp_data=exp_data,
                                            symbol=symbol)
                return plt
            else:
                self._plot_volume_expansion(plt,
                                            exp_data=exp_data,
                                            symbol=symbol)
        else:
            return None

    def plot_pdf_volume_expansion(self,
                                  exp_data=None,
                                  symbol='o',
                                  filename='volume_expansion.pdf'):
        if self._temperatures[self._max_t_index] > 300:
            import matplotlib.pyplot as plt
            plt.rcParams['backend'] = 'PDF'
            plt.rcParams['pdf.fonttype'] = 3
            plt.rcParams['font.family'] = 'serif'
            plt.rcParams['axes.labelsize'] = 18
            plt.rcParams['figure.subplot.left'] = 0.15
            plt.rcParams['figure.subplot.bottom'] = 0.15
            plt.rcParams['figure.figsize'] = 8, 6

            self._plot_volume_expansion(plt,
                                        exp_data=exp_data,
                                        symbol=symbol)
            plt.savefig(filename)
            plt.close()

    def write_volume_expansion(self, filename='volume_expansion.dat'):
        if self._temperatures[self._max_t_index] > 300:
            w = open(filename, 'w')
            for i in range(self._max_t_index):
                w.write("%20.15f %25.15f\n" % (self._temperatures[i],
                                               self._volume_expansions[i]))
            w.close()

    def get_gibbs_temperature(self):
        return self._equiv_energies[:self._max_t_index]

    def plot_gibbs_temperature(self,
                               plt=None,
                               xlabel='Temperature (K)',
                               ylabel='Gibbs free energy'):

        if plt is None:
            import matplotlib.pyplot as _plt
            self._plot_gibbs_temperature(_plt, xlabel=xlabel, ylabel=ylabel)
            return _plt
        else:
            self._plot_gibbs_temperature(plt, xlabel=xlabel, ylabel=ylabel)

    def plot_pdf_gibbs_temperature(self, filename='gibbs-temperature.pdf'):
        import matplotlib.pyplot as plt
        plt.rcParams['backend'] = 'PDF'
        plt.rcParams['pdf.fonttype'] = 42
        plt.rcParams['font.family'] = 'serif'
        plt.rcParams['axes.labelsize'] = 18
        plt.rcParams['figure.subplot.left'] = 0.15
        plt.rcParams['figure.subplot.bottom'] = 0.15
        plt.rcParams['figure.figsize'] = 8, 6

        self._plot_gibbs_temperature(plt)
        plt.savefig(filename)
        plt.close()

    def write_gibbs_temperature(self, filename='gibbs-temperature.dat'):
        w = open(filename, 'w')
        for i in range(self._max_t_index):
            w.write("%20.15f %25.15f\n" % (self._temperatures[i],
                                           self._equiv_energies[i]))
        w.close()

    def get_bulk_modulus_temperature(self):
        return self._equiv_bulk_modulus[:self._max_t_index]

    def plot_bulk_modulus_temperature(self,
                                      plt=None,
                                      xlabel='Temperature (K)',
                                      ylabel='Bulk modulus'):
        if plt is None:
            import matplotlib.pyplot as _plt
            self._plot_bulk_modulus_temperature(_plt,
                                                xlabel=xlabel,
                                                ylabel=ylabel)
            return _plt
        else:
            self._plot_bulk_modulus_temperature(plt,
                                                xlabel=xlabel,
                                                ylabel=ylabel)

    def plot_pdf_bulk_modulus_temperature(
            self,
            filename='bulk_modulus-temperature.pdf'):
        import matplotlib.pyplot as plt
        plt.rcParams['backend'] = 'PDF'
        plt.rcParams['pdf.fonttype'] = 42
        plt.rcParams['font.family'] = 'serif'
        plt.rcParams['axes.labelsize'] = 18
        plt.rcParams['figure.subplot.left'] = 0.15
        plt.rcParams['figure.subplot.bottom'] = 0.15
        plt.rcParams['figure.figsize'] = 8, 6

        self._plot_bulk_modulus_temperature(plt)
        plt.savefig(filename)
        plt.close()

    def write_bulk_modulus_temperature(self,
                                       filename='bulk_modulus-temperature.dat'):
        w = open(filename, 'w')
        for i in range(self._max_t_index):
            w.write("%20.15f %25.15f\n" % (self._temperatures[i],
                                           self._equiv_bulk_modulus[i]))
        w.close()

    def get_heat_capacity_P_numerical(self, exp_data=None):
        return self._cp_numerical[:self._max_t_index]

    def plot_heat_capacity_P_numerical(self, Z=1, exp_data=None, plt=None):
        if plt is None:
            import matplotlib.pyplot as _plt
            self._plot_heat_capacity_P_numerical(_plt, Z=Z, exp_data=exp_data)
            return _plt
        else:
            import matplotlib.pyplot as plt

    def plot_pdf_heat_capacity_P_numerical(self,
                                           exp_data=None,
                                           filename='Cp-temperature.pdf'):
        import matplotlib.pyplot as plt
        plt.rcParams['backend'] = 'PDF'
        plt.rcParams['pdf.fonttype'] = 42
        plt.rcParams['font.family'] = 'serif'
        plt.rcParams['axes.labelsize'] = 18
        plt.rcParams['figure.subplot.left'] = 0.15
        plt.rcParams['figure.subplot.bottom'] = 0.15
        plt.rcParams['figure.figsize'] = 8, 6

        self._plot_heat_capacity_P_numerical(plt, exp_data=exp_data)
        plt.savefig(filename)
        plt.close()

    def write_heat_capacity_P_numerical(self, filename='Cp-temperature.dat'):
        w = open(filename, 'w')
        for i in range(self._max_t_index):
            w.write("%20.15f %20.15f\n" % (self._temperatures[i],
                                           self._cp_numerical[i]))
        w.close()

    def get_heat_capacity_P_polyfit(self):
        return self._cp_polyfit[:self._max_t_index]

    def plot_heat_capacity_P_polyfit(self, Z=1, exp_data=None, plt=None):
        if plt is None:
            import matplotlib.pyplot as _plt
            self._plot_heat_capacity_P_polyfit(_plt, Z=Z, exp_data=exp_data)
            return _plt
        else:
            self._plot_heat_capacity_P_polyfit(plt, Z=Z, exp_data=exp_data)

    def plot_pdf_heat_capacity_P_polyfit(self,
                                         exp_data=None,
                                         filename='Cp-temperature_polyfit.pdf'):
        import matplotlib.pyplot as plt
        plt.rcParams['backend'] = 'PDF'
        plt.rcParams['pdf.fonttype'] = 42
        plt.rcParams['font.family'] = 'serif'
        plt.rcParams['axes.labelsize'] = 18
        plt.rcParams['figure.subplot.left'] = 0.15
        plt.rcParams['figure.subplot.bottom'] = 0.15
        plt.rcParams['figure.figsize'] = 8, 6

        self._plot_heat_capacity_P_polyfit(plt,
                                           exp_data=exp_data)
        plt.savefig(filename)
        plt.close()

    def write_heat_capacity_P_polyfit(self,
                                      filename='Cp-temperature_polyfit.dat',
                                      filename_ev='entropy-volume.dat',
                                      filename_cvv='Cv-volume.dat',
                                      filename_dsdvt='dsdv-temperature.dat'):
        wve = open(filename_ev, 'w')
        wvcv = open(filename_cvv, 'w')
        for i in range(1, self._max_t_index):
            t = self._temperatures[i]
            wve.write("# temperature %20.15f\n" % t)
            wve.write("# %20.15f %20.15f %20.15f %20.15f %20.15f\n" %
                      tuple(self._volume_cv_parameters[i - 1]))
            wvcv.write("# temperature %20.15f\n" % t)
            wvcv.write("# %20.15f %20.15f %20.15f %20.15f %20.15f\n" %
                       tuple(self._volume_entropy_parameters[i - 1]))
            for ve, vcv in zip(self._volume_entropy[i - 1],
                               self._volume_cv[i - 1]):
                wve.write("%20.15f %20.15f\n" % tuple(ve))
                wvcv.write("%20.15f %20.15f\n" % tuple(vcv))
            wve.write("\n\n")
            wvcv.write("\n\n")
        wve.close()
        wvcv.close()

        w = open(filename, 'w')
        for i in range(self._max_t_index):
            w.write("%20.15f %20.15f\n" % (self._temperatures[i],
                                           self._cp_polyfit[i]))
        w.close()

        w = open(filename_dsdvt, 'w') # GPa
        for i in range(self._max_t_index):
            w.write("%20.15f %20.15f\n" % (self._temperatures[i],
                                           self._dsdv[i] * 1e21 / Avogadro))
        w.close()

    def get_gruneisen_temperature(self):
        return self._gruneisen_parameters[:self._max_t_index]

    def plot_gruneisen_temperature(self, plt=None):
        if plt is None:
            import matplotlib.pyplot as _plt
            self._plot_gruneisen_temperature(_plt)
            return _plt
        else:
            self._plot_gruneisen_temperature(plt)

    def plot_pdf_gruneisen_temperature(self,
                                       filename='gruneisen-temperature.pdf'):
        import matplotlib.pyplot as plt
        plt.rcParams['backend'] = 'PDF'
        plt.rcParams['pdf.fonttype'] = 42
        plt.rcParams['font.family'] = 'serif'
        plt.rcParams['axes.labelsize'] = 18
        plt.rcParams['figure.subplot.left'] = 0.15
        plt.rcParams['figure.subplot.bottom'] = 0.15
        plt.rcParams['figure.figsize'] = 8, 6

        self._plot_gruneisen_temperature(plt)
        plt.savefig(filename)
        plt.close()

    def write_gruneisen_temperature(self, filename='gruneisen-temperature.dat'):
        w = open(filename, 'w')
        for i in range(self._max_t_index):
            w.write("%20.15f %25.15f\n" % (self._temperatures[i],
                                           self._gruneisen_parameters[i]))
        w.close()

    def _plot_helmholtz_volume(self,
                               plt,
                               thin_number=10,
                               xlabel=r'Volume $(\AA^3)$',
                               ylabel='Free energy (eV)'):
        volume_points = np.linspace(min(self._volumes),
                                    max(self._volumes),
                                    201)
        selected_volumes = []
        selected_energies = []

        thin_index = 0
        for i, t in enumerate(self._temperatures[:self._max_t_index]):
            if i % thin_number == 0:
                selected_volumes.append(self._equiv_volumes[i])
                selected_energies.append(self._equiv_energies[i])
                plt.plot(self._volumes,
                         self._free_energies[i],
                         'bo', markeredgecolor='b', markersize=3)
                plt.plot(volume_points,
                         self._eos(volume_points,
                                   *self._equiv_parameters[i]), 'b-')
                thin_index = i

        for i, j in enumerate((0, thin_index)):
            plt.text(self._volumes[-2],
                     self._free_energies[j][-1] + (1 - i * 2) * 0.1 - 0.05,
                     "%dK" % int(self._temperatures[j]),
                     fontsize=8)

        plt.plot(selected_volumes,
                 selected_energies,
                 'ro-', markeredgecolor='r', markersize=3)
        plt.xlabel(xlabel)
        plt.ylabel(ylabel)

    def _plot_volume_temperature(self,
                                 plt,
                                 exp_data=None,
                                 xlabel='Temperature (K)',
                                 ylabel=r'Volume $(\AA^3)$'):
        plt.xlabel(xlabel)
        plt.ylabel(ylabel)
        plt.plot(self._temperatures[:self._max_t_index],
                 self._equiv_volumes[:self._max_t_index],
                 'r-')
        # exp
        if exp_data:
            plt.plot(exp_data[0], exp_data[1], 'ro')

    def _plot_thermal_expansion(
            self,
            plt,
            xlabel='Temperature (K)',
            ylabel=r'Thermal expansion $\times 10^6 (\mathrm{K}^{-1})$'):

        beta = np.array(self._thermal_expansions) * 1e6
        plt.plot(self._temperatures[:self._max_t_index],
                 beta[:self._max_t_index],
                 'r-')
        plt.xlabel(xlabel)
        plt.ylabel(ylabel)

    def _plot_volume_expansion(
            self,
            plt,
            exp_data=None,
            symbol='o',
            xlabel='Temperature (K)',
            ylabel=r'Volume expansion $\Delta L/L_0 \, (L=V^{\,1/3})$'):
        plt.plot(self._temperatures[:self._max_t_index],
                 self._volume_expansions[:self._max_t_index],
                 'r-')

        if exp_data:
            plt.plot(exp_data[0],
                     (exp_data[1] / exp_data[1][0]) ** (1.0 / 3) - 1,
                     symbol)

        plt.xlabel(xlabel)
        plt.ylabel(ylabel)
        plt.xlim(self._temperatures[0],
                 self._temperatures[self._max_t_index])

    def _plot_gibbs_temperature(self,
                                plt,
                                xlabel='Temperature (K)',
                                ylabel='Gibbs free energy (eV)'):
        plt.xlabel(xlabel)
        plt.ylabel(ylabel)
        plt.plot(self._temperatures[:self._max_t_index],
                 self._equiv_energies[:self._max_t_index],
                 'r-')

    def _plot_bulk_modulus_temperature(self,
                                       plt,
                                       xlabel='Temperature (K)',
                                       ylabel='Bulk modulus (GPa)'):
        plt.xlabel(xlabel)
        plt.ylabel(ylabel)
        plt.plot(self._temperatures[:self._max_t_index],
                 self._equiv_bulk_modulus[:self._max_t_index],
                 'r-')

    def _plot_heat_capacity_P_numerical(
            self,
            plt,
            Z=1,
            exp_data=None,
            xlabel='Temperature (K)',
            ylabel=r'$C\mathrm{_P}$ $\mathrm{(J/mol\cdot K)}$'):
        plt.xlabel(xlabel)
        plt.ylabel(ylabel)
        plt.plot(self._temperatures[:self._max_t_index],
                 np.array(self._cp_numerical[:self._max_t_index]) / Z,
                 'r-')

        # exp
        if exp_data:
            plt.plot(exp_data[0], exp_data[1], 'ro')

    def _plot_heat_capacity_P_polyfit(
            self,
            plt,
            Z=1,
            exp_data=None,
            xlabel='Temperature (K)',
            ylabel=r'$C\mathrm{_P}$ $\mathrm{(J/mol\cdot K)}$'):
        plt.xlabel(xlabel)
        plt.ylabel(ylabel)
        plt.plot(self._temperatures[:self._max_t_index],
                 np.array(self._cp_polyfit[:self._max_t_index]) / Z,
                 'r-')

        # exp
        if exp_data:
            plt.plot(exp_data[0], exp_data[1], 'ro')

    def _plot_gruneisen_temperature(self,
                                    plt,
                                    xlabel='Temperature (K)',
                                    ylabel='Gruneisen parameter'):
        plt.xlabel(xlabel)
        plt.ylabel(ylabel)
        plt.plot(self._temperatures[:self._max_t_index],
                 self._gruneisen_parameters[:self._max_t_index],
                 'r-')

    def _set_thermal_expansion(self):
        beta = [0.]
        dt = self._temperatures[1] - self._temperatures[0]
        for i in range(self._max_t_index):
            beta.append((self._equiv_volumes[i + 2] - self._equiv_volumes[i]) /
                        (2 * dt) / self._equiv_volumes[i + 1])

        self._thermal_expansions = beta

    def _set_volume_expansion(self):
        if self._temperatures[self._max_t_index] > 300:
            l = np.array(self._equiv_volumes) ** (1.0 / 3)
            for i in range(self._max_t_index):
                t = self._temperatures[i]
                if (abs(t - 300) <
                    (self._temperatures[1] - self._temperatures[0]) / 10):
                    l_0 = (self._equiv_volumes[i]) ** (1.0 / 3)
                    break

            self._volume_expansions = l / l_0 - 1

    def _set_heat_capacity_P_numerical(self):
        cp = []
        g = np.array(self._equiv_energies) * EvTokJmol * 1000
        cp.append(0.0)
        cp.append(0.0)
        dt = self._temperatures[1] - self._temperatures[0]
        for i in range(2, self._max_t_index):
            cp.append(-(g[i + 2] - 2 * g[i] + g[i - 2]) /
                       (dt ** 2) / 4 * self._temperatures[i])
        self._cp_numerical = cp

    def _set_heat_capacity_P_polyfit(self):
        cp = [0.0]
        dsdv = [0.0]
        self._volume_entropy_parameters = []
        self._volume_cv_parameters = []
        self._volume_entropy = []
        self._volume_cv = []

        dt = self._temperatures[1] - self._temperatures[0]
        for j in range(1, self._max_t_index):
            t = self._temperatures[j]
            x = self._equiv_volumes[j]

            parameters = np.polyfit(self._volumes, self._cv[j], 4)
            cv_p = np.dot(parameters, np.array([x**4, x**3, x**2, x, 1]))
            self._volume_cv_parameters.append(parameters)

            parameters = np.polyfit(self._volumes, self._entropy[j], 4)
            dsdv_t = np.dot(parameters[:4], np.array(
                [4 * x**3, 3 * x**2, 2 * x, 1]))
            self._volume_entropy_parameters.append(parameters)

            dvdt = (self._equiv_volumes[j + 1] -
                    self._equiv_volumes[j - 1]) / dt / 2

            cp.append(cv_p + t * dvdt * dsdv_t)
            dsdv.append(dsdv_t)

            self._volume_cv.append(np.array([self._volumes, self._cv[j]]).T)
            self._volume_entropy.append(np.array([self._volumes,
                                                  self._entropy[j]]).T)

        self._cp_polyfit = cp
        self._dsdv = dsdv

    def _set_gruneisen_parameter(self):
        gamma = [0]
        for i in range(1, self._max_t_index):
            v = self._equiv_volumes[i]
            kt = self._equiv_bulk_modulus[i]
            beta = self._thermal_expansions[i]
            parameters = np.polyfit(self._volumes, self._cv[i], 4)
            cv = (np.dot(parameters, [v**4, v**3, v**2, v, 1]) /
                  v / 1000 / EvTokJmol * EVAngstromToGPa)
            if cv < 1e-10:
                gamma.append(0.0)
            else:
                gamma.append(beta * kt / cv)
        self._gruneisen_parameters = gamma

    def _get_max_t_index(self, temperatures):
        if self._t_max is None:
            return len(self._all_temperatures) - 3
        else:
            max_t_index = 0

            for i, t in enumerate(temperatures):
                if self._t_max + 1e-5 < t:
                    max_t_index = i + 1
                    break

            if (max_t_index > len(temperatures) - 3 or
                max_t_index < 2):
                max_t_index = len(temperatures) - 3

            return max_t_index



# Copyright (C) 2011 Atsushi Togo
# All rights reserved.
#
# This file is part of phonopy.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
# * Neither the name of the phonopy project nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import numpy as np
from phonopy.structure.atoms import PhonopyAtoms as Atoms

def get_supercell(unitcell, supercell_matrix, symprec=1e-5):
    return Supercell(unitcell, supercell_matrix, symprec=symprec)

def get_primitive(supercell, primitive_frame, symprec=1e-5):
    return Primitive(supercell, primitive_frame, symprec=symprec)

def trim_cell(relative_axes, cell, symprec):
    """
    relative_axes: relative axes to supercell axes
    Trim positions outside relative axes
    
    """
    positions = cell.get_scaled_positions()
    numbers = cell.get_atomic_numbers()
    masses = cell.get_masses()
    magmoms = cell.get_magnetic_moments()    
    lattice = cell.get_cell()
    trimed_lattice = np.dot(relative_axes.T, lattice)

    trimed_positions = []
    trimed_numbers = []
    if masses is None:
        trimed_masses = None
    else:
        trimed_masses = []
    if magmoms is None:
        trimed_magmoms = None
    else:
        trimed_magmoms = []
    extracted_atoms = []

    positions_in_new_lattice = np.dot(positions, np.linalg.inv(relative_axes).T)
    positions_in_new_lattice -= np.floor(positions_in_new_lattice)
    trimed_positions = np.zeros_like(positions_in_new_lattice)
    num_atom = 0
    
    mapping_table = np.arange(len(positions), dtype='intc')
    symprec2 = symprec ** 2
    for i, pos in enumerate(positions_in_new_lattice):
        is_overlap = False
        if num_atom > 0:
            diff = trimed_positions[:num_atom] - pos
            diff -= np.rint(diff)
            # Older numpy doesn't support axis argument.
            # distances = np.linalg.norm(np.dot(diff, trimed_lattice), axis=1)
            # overlap_indices = np.where(distances < symprec)[0]
            distances2 = np.sum(np.dot(diff, trimed_lattice) ** 2, axis=1)
            overlap_indices = np.where(distances2 < symprec2)[0]
            if len(overlap_indices) > 0:
                is_overlap = True
                mapping_table[i] = extracted_atoms[overlap_indices[0]]

        if not is_overlap:
            trimed_positions[num_atom] = pos
            num_atom += 1
            trimed_numbers.append(numbers[i])
            if masses is not None:
                trimed_masses.append(masses[i])
            if magmoms is not None:
                trimed_magmoms.append(magmoms[i])
            extracted_atoms.append(i)

    trimed_cell = Atoms(numbers=trimed_numbers,
                        masses=trimed_masses,
                        magmoms=trimed_magmoms,
                        scaled_positions=trimed_positions[:num_atom],
                        cell=trimed_lattice,
                        pbc=True)

    return trimed_cell, extracted_atoms, mapping_table

def print_cell(cell, mapping=None, stars=None):
    symbols = cell.get_chemical_symbols()
    masses = cell.get_masses()
    magmoms = cell.get_magnetic_moments()
    lattice = cell.get_cell()
    print("Lattice vectors:")
    print("  a %20.15f %20.15f %20.15f" % tuple(lattice[0]))
    print("  b %20.15f %20.15f %20.15f" % tuple(lattice[1]))
    print("  c %20.15f %20.15f %20.15f" % tuple(lattice[2]))
    print("Atomic positions (fractional):")
    for i, v in enumerate(cell.get_scaled_positions()):
        num = " "
        if stars is not None:
            if i in stars:
                num = "*"
        num += "%d" % (i + 1)
        line = ("%5s %-2s%18.14f%18.14f%18.14f" % 
                (num, symbols[i], v[0], v[1], v[2]))
        if masses is not None:
            line += " %7.3f" % masses[i]
        if magmoms is not None:
            line += "  %5.3f" % magmoms[i]
        if mapping is None:
            print(line)
        else:
            print(line + " > %d" % (mapping[i] + 1))

class Supercell(Atoms):
    """Build supercell from supercell matrix
    In this function, unit cell is considered 
    [1,0,0]
    [0,1,0]
    [0,0,1].
    Supercell matrix is given by relative ratio, e.g,
    [-1, 1, 1]
    [ 1,-1, 1]  is for FCC from simple cubic.
    [ 1, 1,-1]
    In this case multiplicities of surrounding simple lattice are [2,2,2].

    First, create supercell with surrounding simple lattice.
    Second, trim the surrounding supercell with the target lattice.
    """
    
    def __init__(self, unitcell, supercell_matrix, symprec=1e-5):
        self._s2u_map = None
        self._u2s_map = None
        self._u2u_map = None
        self._supercell_matrix = np.array(supercell_matrix, dtype='intc')
        self._create_supercell(unitcell, symprec)

    def get_supercell_matrix(self):
        return self._supercell_matrix
        
    def get_supercell_to_unitcell_map(self):
        return self._s2u_map

    def get_unitcell_to_supercell_map(self):
        return self._u2s_map

    def get_unitcell_to_unitcell_map(self):
        return self._u2u_map

    def _create_supercell(self, unitcell, symprec):
        mat = self._supercell_matrix
        frame = self._get_surrounding_frame(mat)
        sur_cell, u2sur_map = self._get_simple_supercell(frame, unitcell)
    
        # Trim the simple supercell by the supercell matrix
        trim_frame = np.array([mat[0] / float(frame[0]),
                               mat[1] / float(frame[1]),
                               mat[2] / float(frame[2])])
        supercell, sur2s_map, mapping_table = trim_cell(trim_frame,
                                                        sur_cell,
                                                        symprec)

        multi = supercell.get_number_of_atoms() // unitcell.get_number_of_atoms()
        
        if multi != determinant(self._supercell_matrix):
            print("Supercell creation failed.")
            print("Probably some atoms are overwrapped. "
                  "The mapping table is give below.")
            print(mapping_table)
            Atoms.__init__(self)
        else:            
            Atoms.__init__(self,
                           numbers=supercell.get_atomic_numbers(),
                           masses=supercell.get_masses(),
                           magmoms=supercell.get_magnetic_moments(),
                           scaled_positions=supercell.get_scaled_positions(),
                           cell=supercell.get_cell(),
                           pbc=True)
            self._u2s_map = np.arange(unitcell.get_number_of_atoms()) * multi
            self._u2u_map = dict([(j, i) for i, j in enumerate(self._u2s_map)])
            self._s2u_map = np.array(u2sur_map)[sur2s_map] * multi

    def _get_surrounding_frame(self, supercell_matrix):
        # Build a frame surrounding supercell lattice
        # For example,
        #  [2,0,0]
        #  [0,2,0] is the frame for FCC from simple cubic.
        #  [0,0,2]
        m = np.array(supercell_matrix)
        axes = np.array([[0, 0, 0],
                         m[:,0],
                         m[:,1],
                         m[:,2],
                         m[:,1] + m[:,2],
                         m[:,2] + m[:,0],
                         m[:,0] + m[:,1],
                         m[:,0] + m[:,1] + m[:,2]])
        frame = [max(axes[:,i]) - min(axes[:,i]) for i in (0,1,2)]
        return frame

    def _get_simple_supercell(self, multi, unitcell):
        # Scaled positions within the frame, i.e., create a supercell that
        # is made simply to multiply the input cell.
        positions = unitcell.get_scaled_positions()
        numbers = unitcell.get_atomic_numbers()
        masses = unitcell.get_masses()
        magmoms = unitcell.get_magnetic_moments()
        lattice = unitcell.get_cell()

        atom_map = []
        positions_multi = []
        numbers_multi = []
        if masses is None:
            masses_multi = None
        else:
            masses_multi = []
        if magmoms is None:
            magmoms_multi = None
        else:
            magmoms_multi = []
        for l, pos in enumerate(positions):
            for i in range(multi[2]):
                for j in range(multi[1]):
                    for k in range(multi[0]):
                        positions_multi.append([(pos[0] + k) / multi[0],
                                                (pos[1] + j) / multi[1],
                                                (pos[2] + i) / multi[2]])
                        numbers_multi.append(numbers[l])
                        if masses is not None:
                            masses_multi.append(masses[l])
                        atom_map.append(l)
                        if magmoms is not None:
                            magmoms_multi.append(magmoms[l])

        simple_supercell = Atoms(numbers=numbers_multi,
                                 masses=masses_multi,
                                 magmoms=magmoms_multi,
                                 scaled_positions=positions_multi,
                                 cell=np.dot(np.diag(multi), lattice),
                                 pbc=True)

        return simple_supercell, atom_map

class Primitive(Atoms):
    def __init__(self, supercell, primitive_matrix, symprec=1e-5):
        """
        primitive_matrix (3x3 matrix):
        Primitive lattice is given with respect to supercell by
           np.dot(primitive_matrix.T, supercell.get_cell())
        """
        self._primitive_matrix = np.array(primitive_matrix)
        self._symprec = symprec
        self._p2s_map = None
        self._s2p_map = None
        self._p2p_map = None
        self._primitive_cell(supercell)
        self._supercell_to_primitive_map(supercell.get_scaled_positions())
        self._primitive_to_primitive_map()

    def get_primitive_matrix(self):
        return self._primitive_matrix
        
    def get_primitive_to_supercell_map(self):
        return self._p2s_map

    def get_supercell_to_primitive_map(self):
        return self._s2p_map

    def get_primitive_to_primitive_map(self):
        return self._p2p_map

    def _primitive_cell(self, supercell):
        trimed_cell, p2s_map, mapping_table = trim_cell(self._primitive_matrix,
                                                        supercell,
                                                        self._symprec)
        Atoms.__init__(self,
                       numbers=trimed_cell.get_atomic_numbers(),
                       masses=trimed_cell.get_masses(),
                       magmoms=trimed_cell.get_magnetic_moments(),
                       scaled_positions=trimed_cell.get_scaled_positions(),
                       cell=trimed_cell.get_cell(),
                       pbc=True)

        self._p2s_map = np.array(p2s_map, dtype='intc')

    def _supercell_to_primitive_map(self, pos):
        inv_F = np.linalg.inv(self._primitive_matrix)
        s2p_map = []
        for i in range(pos.shape[0]):
            s_pos = np.dot(pos[i], inv_F.T)
            for j in self._p2s_map:
                p_pos = np.dot(pos[j], inv_F.T)
                diff = p_pos - s_pos
                diff -= np.rint(diff)
                if (abs(diff) < self._symprec).all():
                    s2p_map.append(j)
                    break
        self._s2p_map = np.array(s2p_map, dtype='intc')

    def _primitive_to_primitive_map(self):
        """
        Mapping table from supercell index to primitive index
        in primitive cell
        """
        self._p2p_map = dict([(j, i) for i, j in enumerate(self._p2s_map)])

#
# Get distance between a pair of atoms
#
def get_distance(cell, a0, a1, tolerance=1e-5):
    """
    Return the shortest distance between a pair of atoms in PBC
    """
    reduced_bases = get_reduced_bases(cell.get_cell(), tolerance)
    scaled_pos = np.dot(cell.get_positions(), np.linalg.inv(reduced_bases))
    # move scaled atomic positions into -0.5 < r <= 0.5
    for pos in scaled_pos:
        pos -= np.rint(pos)

    # Look for the shortest one in surrounded 3x3x3 cells 
    distances = []
    for i in (-1, 0, 1):
        for j in (-1, 0, 1):
            for k in (-1, 0, 1):
                distances.append(np.linalg.norm(
                        np.dot(scaled_pos[a0] - scaled_pos[a1] + [i, j, k],
                               reduced_bases)))
    return min(distances)

#
# Delaunay reduction
#    
def get_reduced_bases(lattice, tolerance=1e-5):
    """
    This is an implementation of Delaunay reduction.
    Some information is found in International table.

    lattice: row vectors
    return lattice: row vectors
    """
    return get_Delaunay_reduction(lattice, tolerance)

def get_Delaunay_reduction(lattice, tolerance):
    extended_bases = np.zeros((4, 3), dtype='double')
    extended_bases[:3, :] = lattice
    extended_bases[3] = -np.sum(lattice, axis=0)

    for i in range(100):
        if reduce_bases(extended_bases, tolerance):
            break
    if i == 99:
        print("Delaunary reduction was failed.")

    shortest = get_shortest_bases_from_extented_bases(extended_bases, tolerance)

    return shortest

def reduce_bases(extended_bases, tolerance):
    metric = np.dot(extended_bases, extended_bases.T)
    for i in range(4):
        for j in range(i+1, 4):
            if metric[i][j] > tolerance:
                for k in range(4):
                    if (k != i) and (k != j):
                        extended_bases[k] += extended_bases[i]
                extended_bases[i] = -extended_bases[i]
                extended_bases[j] = extended_bases[j]
                return False

    # Reduction is completed.
    # All non diagonal elements of metric tensor is negative.
    return True

def get_shortest_bases_from_extented_bases(extended_bases, tolerance):

    def mycmp(x, y):
        return cmp(np.vdot(x, x), np.vdot(y, y))

    basis = np.zeros((7, 3), dtype='double')
    basis[:4] = extended_bases
    basis[4]  = extended_bases[0] + extended_bases[1]
    basis[5]  = extended_bases[1] + extended_bases[2]
    basis[6]  = extended_bases[2] + extended_bases[0]
    # Sort bases by the lengthes (shorter is earlier)
    basis = sorted(basis, key=lambda vec: (vec ** 2).sum())
    
    # Choose shortest and linearly independent three bases
    # This algorithm may not be perfect.
    for i in range(7):
        for j in range(i + 1, 7):
            for k in range(j + 1, 7):
                if abs(np.linalg.det(
                        [basis[i], basis[j], basis[k]])) > tolerance:
                    return np.array([basis[i], basis[j], basis[k]])

    print("Delaunary reduction is failed.")
    return np.array(basis[:3], dtype='double')

#
# Other tiny tools
#    
def get_angles(lattice):
    a, b, c = get_cell_parameters(lattice)
    alpha = np.arccos(np.vdot(lattice[1], lattice[2]) / b / c) / np.pi * 180
    beta  = np.arccos(np.vdot(lattice[2], lattice[0]) / c / a) / np.pi * 180
    gamma = np.arccos(np.vdot(lattice[0], lattice[1]) / a / b) / np.pi * 180
    return alpha, beta, gamma

def get_cell_parameters(lattice):
    return np.sqrt(np.dot (lattice, lattice.transpose()).diagonal())

def get_cell_matrix(a, b, c, alpha, beta, gamma):
    # These follow 'matrix_lattice_init' in matrix.c of GDIS
    alpha *= np.pi / 180
    beta *= np.pi / 180
    gamma *= np.pi / 180
    a1 = a
    a2 = 0.0
    a3 = 0.0
    b1 = np.cos(gamma)
    b2 = np.sin(gamma)
    b3 = 0.0
    c1 = np.cos(beta)
    c2 = (2 * np.cos(alpha) + b1**2 + b2**2 - 2 * b1 * c1 - 1) / (2 * b2)
    c3 = np.sqrt(1 - c1**2 - c2**2)
    lattice = np.zeros((3, 3), dtype='double')
    lattice[0, 0] = a
    lattice[1] = np.array([b1, b2, b3]) * b
    lattice[2] = np.array([c1, c2, c3]) * c
    return lattice

def determinant(m):
    return (m[0][0] * m[1][1] * m[2][2] -
            m[0][0] * m[1][2] * m[2][1] +
            m[0][1] * m[1][2] * m[2][0] -
            m[0][1] * m[1][0] * m[2][2] +
            m[0][2] * m[1][0] * m[2][1] -
            m[0][2] * m[1][1] * m[2][0])


